/*
 * #%L
 * ImgLib2: a general-purpose, multidimensional image processing library.
 * %%
 * Copyright (C) 2009 - 2013 Stephan Preibisch, Tobias Pietzsch, Barry DeZonia,
 * Stephan Saalfeld, Albert Cardona, Curtis Rueden, Christian Dietz, Jean-Yves
 * Tinevez, Johannes Schindelin, Lee Kamentsky, Larry Lindsey, Grant Harris,
 * Mark Hiner, Aivar Grislis, Martin Horn, Nick Perry, Michael Zinsmaier,
 * Steffen Jaensch, Jan Funke, Mark Longair, and Dimiter Prodanov.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

package net.imglib2.nearestneighbor;
/**
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.  Redistributions in binary
 * form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials
 * provided with the distribution.  Neither the name of the Fiji project nor
 * the names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.ImageStack;
import mpicbg.util.Timer;
import net.imglib2.Cursor;
import net.imglib2.IterableInterval;
import net.imglib2.RealPoint;
import net.imglib2.collection.KDTree;
import net.imglib2.collection.RealPointSampleList;
import net.imglib2.exception.ImgLibException;
import net.imglib2.img.imageplus.ImagePlusImg;
import net.imglib2.img.imageplus.ImagePlusImgFactory;
import net.imglib2.interpolation.neighborsearch.InverseDistanceWeightingInterpolator;
import net.imglib2.interpolation.neighborsearch.InverseDistanceWeightingInterpolatorFactory;
import net.imglib2.neighborsearch.KNearestNeighborSearch;
import net.imglib2.neighborsearch.KNearestNeighborSearchOnIterableRealInterval;
import net.imglib2.neighborsearch.KNearestNeighborSearchOnKDTree;
import net.imglib2.neighborsearch.NearestNeighborSearch;
import net.imglib2.neighborsearch.NearestNeighborSearchOnIterableRealInterval;
import net.imglib2.neighborsearch.NearestNeighborSearchOnKDTree;
import net.imglib2.type.Type;
import net.imglib2.type.numeric.RealType;
import net.imglib2.type.numeric.integer.UnsignedByteType;

/**
 * TODO
 *
 * @author Stephan Preibisch
 * @author Stephan Saalfeld
 */
public class KNearestNeighborSearchBehavior
{
	final static private long[] size = new long[]{ 200, 164 };
//	final static private long[] size = new long[]{ 640, 395 };
	final static private double[][] coordinates =
			new double[][]
					{ { 51, 52 }, { 41, 36 }, { 55, 64 }, { 175, 18 }, { 101, 97 }, { 117, 126 }, { 190, 66 }, { 12, 148 }, { 187, 139 }, { 132, 143 }, { 1, 19 }, { 192, 160 }, { 28, 21 }, { 25, 31 }, { 71, 143 }, { 129, 61 }, { 101, 25 }, { 103, 91 }, { 152, 51 }, { 10, 24 }, { 196, 122 }, { 99, 162 }, { 118, 0 }, { 12, 139 }, { 150, 100 }, { 174, 140 }, { 54, 24 }, { 180, 115 }, { 158, 127 }, { 194, 141 }, { 94, 163 }, { 153, 158 }, { 183, 107 }, { 170, 1 }, { 190, 27 }, { 76, 25 }, { 188, 94 }, { 49, 160 }, { 152, 107 }, { 3, 96 }, { 32, 88 }, { 75, 12 }, { 9, 95 }, { 198, 36 }, { 87, 75 }, { 153, 67 }, { 151, 109 }, { 115, 138 }, { 174, 149 }, { 150, 128 }, { 105, 57 }, { 96, 9 }, { 106, 85 }, { 170, 118 }, { 44, 13 }, { 132, 134 }, { 116, 20 }, { 186, 79 }, { 84, 39 }, { 104, 109 }, { 12, 7 }, { 13, 4 }, { 175, 133 }, { 175, 73 }, { 88, 130 }, { 59, 38 }, { 23, 47 }, { 126, 143 }, { 99, 115 }, { 38, 138 }, { 133, 4 }, { 182, 44 }, { 127, 136 }, { 138, 41 }, { 56, 160 }, { 7, 153 }, { 38, 96 }, { 198, 145 }, { 54, 162 }, { 5, 14 }, { 26, 126 }, { 97, 126 }, { 184, 155 }, { 157, 151 }, { 58, 76 }, { 128, 129 }, { 167, 125 }, { 37, 144 }, { 14, 113 }, { 148, 67 }, { 187, 141 }, { 49, 158 }, { 23, 162 }, { 70, 82 }, { 72, 133 }, { 146, 135 }, { 50, 32 }, { 69, 19 }, { 88, 78 }, { 79, 129 }, { 84, 156 }, { 180, 121 }, { 38, 47 }, { 165, 23 }, { 52, 101 }, { 183, 72 }, { 7, 159 }, { 32, 6 }, { 141, 69 }, { 35, 83 }, { 16, 62 }, { 51, 44 }, { 163, 137 }, { 92, 88 }, { 88, 133 }, { 51, 23 }, { 183, 49 }, { 125, 80 }, { 58, 120 }, { 109, 117 }, { 140, 117 }, { 128, 106 }, { 114, 74 }, { 179, 65 }, { 17, 45 }, { 36, 116 }, { 121, 131 }, { 185, 74 }, { 181, 156 }, { 24, 56 }, { 126, 102 }, { 172, 153 }, { 84, 146 }, { 161, 140 }, { 136, 21 }, { 181, 130 }, { 145, 160 }, { 43, 112 }, { 129, 162 }, { 133, 114 }, { 99, 131 }, { 101, 144 }, { 67, 31 }, { 24, 54 }, { 113, 45 }, { 104, 135 }, { 180, 46 }, { 156, 86 }, { 88, 66 }, { 6, 20 }, { 19, 132 }, { 9, 94 }, { 129, 147 }, { 156, 108 }, { 90, 38 }, { 160, 92 }, { 5, 10 }, { 114, 97 }, { 100, 3 }, { 37, 31 }, { 163, 139 }, { 135, 68 }, { 109, 111 }, { 150, 154 }, { 198, 108 }, { 127, 59 }, { 199, 158 }, { 171, 134 }, { 161, 109 }, { 87, 28 }, { 137, 37 }, { 54, 115 }, { 188, 36 }, { 157, 0 }, { 111, 88 }, { 114, 23 }, { 38, 19 }, { 178, 84 }, { 39, 113 }, { 198, 25 }, { 130, 84 }, { 97, 86 }, { 63, 47 }, { 196, 115 }, { 79, 16 }, { 138, 84 }, { 116, 95 }, { 147, 107 }, { 155, 68 }, { 66, 9 }, { 125, 141 }, { 134, 139 }, { 44, 86 }, { 68, 2 }, { 88, 87 }, { 177, 139 }, { 152, 77 }, { 113, 23 }, { 54, 71 }, { 47, 52 }, { 41, 5 }, { 105, 159 }, { 143, 12 }, { 158, 78 }, { 23, 46 }, { 23, 88 }, { 81, 81 }, { 184, 101 }, { 195, 115 }, { 114, 108 }, { 136, 44 }, { 190, 23 }, { 15, 63 }, { 191, 85 }, { 94, 136 }, { 173, 20 }, { 40, 159 }, { 74, 86 }, { 10, 43 }, { 58, 50 }, { 134, 97 }, { 84, 51 }, { 162, 108 }, { 95, 20 }, { 40, 138 }, { 51, 60 }, { 75, 28 }, { 62, 87 }, { 75, 45 }, { 47, 42 }, { 45, 145 }, { 26, 55 }, { 34, 45 }, { 168, 32 }, { 137, 38 }, { 155, 78 }, { 4, 158 }, { 91, 83 }, { 172, 156 }, { 160, 85 }, { 194, 0 }, { 196, 82 }, { 99, 160 }, { 188, 56 }, { 183, 118 }, { 27, 86 }, { 31, 53 }, { 75, 118 }, { 48, 80 }, { 49, 87 }, { 88, 6 }, { 3, 26 }, { 164, 44 }, { 41, 12 }, { 187, 38 }, { 6, 26 }, { 51, 161 }, { 97, 50 }, { 182, 74 }, { 164, 104 }, { 56, 54 }, { 184, 109 }, { 168, 58 }, { 83, 96 }, { 89, 35 }, { 95, 45 }, { 196, 75 }, { 72, 130 }, { 99, 136 }, { 122, 82 }, { 140, 150 }, { 53, 49 }, { 32, 39 }, { 190, 60 }, { 111, 95 }, { 163, 130 }, { 146, 71 }, { 170, 111 }, { 173, 30 }, { 70, 145 }, { 167, 87 }, { 168, 87 }, { 87, 110 }, { 62, 151 }, { 61, 34 }, { 50, 116 }, { 154, 148 }, { 130, 110 }, { 36, 129 }, { 135, 47 }, { 106, 3 }, { 177, 58 }, { 53, 27 }, { 145, 87 }, { 121, 41 }, { 143, 75 }, { 150, 104 }, { 23, 9 }, { 120, 68 }, { 5, 97 }, { 133, 88 }, { 56, 96 }, { 144, 103 }, { 45, 45 }, { 168, 4 }, { 195, 99 }, { 4, 76 }, { 168, 10 }, { 53, 66 }, { 144, 104 }, { 176, 133 }, { 36, 132 }, { 78, 82 }, { 143, 73 }, { 146, 5 }, { 112, 121 }, { 135, 76 }, { 92, 90 }, { 44, 10 }, { 132, 119 }, { 124, 95 }, { 175, 25 }, { 198, 121 }, { 60, 148 }, { 14, 136 }, { 78, 121 }, { 182, 136 }, { 147, 113 }, { 4, 27 }, { 19, 103 }, { 57, 78 }, { 18, 3 }, { 108, 148 }, { 114, 51 }, { 153, 70 }, { 80, 72 }, { 73, 35 }, { 72, 134 }, { 142, 0 }, { 112, 15 }, { 99, 109 }, { 23, 93 }, { 7, 102 }, { 72, 163 }, { 76, 136 }, { 167, 9 }, { 144, 96 }, { 30, 159 }, { 47, 54 }, { 147, 12 }, { 185, 106 }, { 4, 131 }, { 1, 78 }, { 49, 145 }, { 154, 76 }, { 23, 42 }, { 135, 60 }, { 102, 61 }, { 74, 105 }, { 180, 38 }, { 161, 96 }, { 102, 124 }, { 63, 29 }, { 39, 11 }, { 186, 22 }, { 58, 110 }, { 38, 78 }, { 162, 17 }, { 179, 78 }, { 166, 146 }, { 123, 143 }, { 99, 13 }, { 171, 36 }, { 10, 57 }, { 175, 13 }, { 64, 124 }, { 83, 35 }, { 56, 144 }, { 160, 163 }, { 88, 119 }, { 22, 128 }, { 82, 157 }, { 70, 3 }, { 10, 7 }, { 10, 35 }, { 109, 36 }, { 23, 105 }, { 168, 142 }, { 196, 7 }, { 80, 91 }, { 163, 127 }, { 28, 103 }, { 138, 44 }, { 163, 104 }, { 18, 86 }, { 50, 39 }, { 92, 22 }, { 80, 112 }, { 105, 107 }, { 179, 31 }, { 3, 149 }, { 134, 144 }, { 84, 71 }, { 72, 2 }, { 49, 21 }, { 97, 146 }, { 18, 17 }, { 75, 84 }, { 1, 139 }, { 107, 14 }, { 52, 12 }, { 162, 113 }, { 104, 87 }, { 117, 146 }, { 68, 103 }, { 181, 34 }, { 16, 22 }, { 124, 107 }, { 25, 124 }, { 141, 58 }, { 138, 125 }, { 62, 105 }, { 7, 8 }, { 197, 37 }, { 73, 63 }, { 12, 150 }, { 105, 148 }, { 122, 20 }, { 139, 27 }, { 112, 75 }, { 121, 59 }, { 101, 99 }, { 184, 23 }, { 22, 25 }, { 187, 13 }, { 173, 21 }, { 83, 89 }, { 44, 136 }, { 184, 113 }, { 70, 156 }, { 6, 10 }, { 171, 91 }, { 166, 55 }, { 70, 149 }, { 24, 133 }, { 69, 26 }, { 2, 26 }, { 173, 75 }, { 184, 12 }, { 71, 14 }, { 72, 66 }, { 162, 37 }, { 184, 35 }, { 126, 120 }, { 190, 147 }, { 170, 43 }, { 188, 10 }, { 91, 70 }, { 127, 12 }, { 178, 48 }, { 186, 123 }, { 81, 147 }, { 84, 64 }, { 81, 78 }, { 170, 57 }, { 75, 95 }, { 2, 56 }, { 49, 56 }, { 165, 50 }, { 48, 24 }, { 72, 75 }, { 69, 78 }, { 160, 141 }, { 73, 25 }, { 95, 132 }, { 58, 37 }, { 71, 120 }, { 159, 123 }, { 98, 54 }, { 95, 48 }, { 156, 78 }, { 135, 83 }, { 77, 63 }, { 55, 23 }, { 111, 17 }, { 151, 108 }, { 96, 84 }, { 19, 163 }, { 100, 17 }, { 16, 98 }, { 11, 82 }, { 106, 155 }, { 130, 111 }, { 123, 13 }, { 105, 20 }, { 58, 80 }, { 145, 155 }, { 158, 90 }, { 15, 154 }, { 134, 98 }, { 17, 162 }, { 46, 121 }, { 171, 147 }, { 168, 148 }, { 78, 96 }, { 196, 73 }, { 135, 63 }, { 109, 31 }, { 128, 86 }, { 58, 89 }, { 61, 13 }, { 78, 152 }, { 116, 84 }, { 29, 100 }, { 186, 98 }, { 35, 64 }, { 54, 72 }, { 114, 126 }, { 21, 107 }, { 63, 43 }, { 25, 52 }, { 186, 16 }, { 126, 79 }, { 129, 113 }, { 171, 108 }, { 112, 48 }, { 161, 14 }, { 30, 54 }, { 124, 5 }, { 167, 86 }, { 138, 114 }, { 18, 63 }, { 84, 155 }, { 182, 139 }, { 193, 97 }, { 62, 78 }, { 1, 51 }, { 85, 29 }, { 189, 137 }, { 88, 61 }, { 113, 113 }, { 79, 152 }, { 18, 118 }, { 40, 31 }, { 170, 114 }, { 16, 103 }, { 173, 23 }, { 22, 29 }, { 39, 30 }, { 198, 20 }, { 132, 161 }, { 151, 133 }, { 59, 48 }, { 154, 43 }, { 64, 153 }, { 191, 43 }, { 76, 120 }, { 78, 39 }, { 140, 110 }, { 86, 150 }, { 19, 68 }, { 133, 7 }, { 5, 8 }, { 53, 67 }, { 153, 75 }, { 119, 134 }, { 41, 38 }, { 79, 34 }, { 89, 123 }, { 148, 47 }, { 74, 108 }, { 122, 106 }, { 42, 83 }, { 94, 105 }, { 178, 33 }, { 133, 137 }, { 180, 77 }, { 25, 146 }, { 22, 6 }, { 162, 65 }, { 15, 9 }, { 77, 136 }, { 13, 40 }, { 19, 95 }, { 113, 143 }, { 112, 103 }, { 0, 74 }, { 85, 147 }, { 70, 157 }, { 125, 144 }, { 81, 84 }, { 2, 138 }, { 119, 62 }, { 65, 121 }, { 141, 148 }, { 144, 22 }, { 22, 112 }, { 2, 82 }, { 68, 41 }, { 109, 61 }, { 54, 126 }, { 177, 77 }, { 21, 102 }, { 140, 102 }, { 6, 94 }, { 69, 10 }, { 39, 2 }, { 146, 8 }, { 48, 151 }, { 142, 79 }, { 149, 107 }, { 129, 80 }, { 4, 157 }, { 49, 142 }, { 106, 40 }, { 164, 115 }, { 51, 28 }, { 55, 128 }, { 99, 143 }, { 25, 2 }, { 70, 138 }, { 1, 68 }, { 89, 56 }, { 32, 9 }, { 115, 17 }, { 109, 62 }, { 171, 116 }, { 188, 17 }, { 175, 108 }, { 10, 9 }, { 198, 156 }, { 79, 82 }, { 35, 89 }, { 8, 150 }, { 38, 125 }, { 39, 105 }, { 96, 61 }, { 82, 70 }, { 76, 142 }, { 157, 134 }, { 125, 125 }, { 183, 137 }, { 22, 163 }, { 0, 138 }, { 29, 48 }, { 19, 159 }, { 68, 95 }, { 180, 97 }, { 185, 17 }, { 172, 15 }, { 81, 89 }, { 156, 79 }, { 171, 82 }, { 182, 56 }, { 114, 137 }, { 35, 143 }, { 53, 114 }, { 146, 106 }, { 69, 98 }, { 10, 152 }, { 154, 61 }, { 76, 18 }, { 194, 106 }, { 184, 59 }, { 175, 83 }, { 187, 136 }, { 68, 84 }, { 74, 107 }, { 132, 117 }, { 141, 61 }, { 111, 140 }, { 27, 80 }, { 25, 58 }, { 157, 42 }, { 177, 22 }, { 2, 96 }, { 27, 131 }, { 191, 111 }, { 34, 106 }, { 192, 40 }, { 174, 67 }, { 120, 64 }, { 81, 51 }, { 10, 75 }, { 80, 11 }, { 128, 21 }, { 55, 50 }, { 1, 155 }, { 53, 154 }, { 149, 77 }, { 83, 134 }, { 79, 26 }, { 48, 152 }, { 32, 63 }, { 176, 105 }, { 127, 147 }, { 17, 13 }, { 188, 111 }, { 0, 64 }, { 2, 28 }, { 198, 97 }, { 15, 139 }, { 59, 78 }, { 1, 9 }, { 5, 82 }, { 174, 96 }, { 109, 45 }, { 32, 15 }, { 54, 151 }, { 187, 137 }, { 19, 148 }, { 188, 18 }, { 163, 147 }, { 192, 58 }, { 156, 139 }, { 55, 110 }, { 129, 158 }, { 147, 2 }, { 169, 62 }, { 89, 117 }, { 198, 150 }, { 130, 132 }, { 6, 98 }, { 22, 110 }, { 122, 163 }, { 62, 162 }, { 57, 37 }, { 100, 119 }, { 160, 107 }, { 129, 136 }, { 128, 113 }, { 35, 59 }, { 100, 150 }, { 85, 54 }, { 77, 105 }, { 141, 86 }, { 27, 19 }, { 37, 59 }, { 40, 56 }, { 188, 28 }, { 135, 160 }, { 105, 43 }, { 187, 14 }, { 3, 142 }, { 89, 50 }, { 34, 93 }, { 170, 78 }, { 29, 49 }, { 146, 53 }, { 113, 30 }, { 97, 64 }, { 130, 130 }, { 70, 45 }, { 65, 114 }, { 67, 127 }, { 98, 58 }, { 193, 25 }, { 105, 122 }, { 65, 54 }, { 17, 156 }, { 75, 82 }, { 97, 106 }, { 47, 25 }, { 187, 1 }, { 6, 72 }, { 65, 40 }, { 3, 125 }, { 14, 102 }, { 196, 131 }, { 7, 107 }, { 199, 3 }, { 100, 6 }, { 19, 151 }, { 63, 152 }, { 4, 101 }, { 184, 89 }, { 10, 17 }, { 142, 37 }, { 59, 143 }, { 98, 132 }, { 102, 159 }, { 49, 43 }, { 193, 102 }, { 193, 140 }, { 110, 32 }, { 65, 55 }, { 7, 138 }, { 67, 55 }, { 131, 162 }, { 98, 154 }, { 139, 142 }, { 199, 68 }, { 27, 151 }, { 174, 88 }, { 117, 1 }, { 146, 13 }, { 41, 7 }, { 173, 83 }, { 92, 123 }, { 179, 85 }, { 119, 41 }, { 189, 46 }, { 191, 34 }, { 120, 110 }, { 199, 154 }, { 151, 64 }, { 115, 93 }, { 85, 16 }, { 4, 48 }, { 195, 149 }, { 172, 111 }, { 2, 98 }, { 64, 112 }, { 166, 83 }, { 189, 155 }, { 104, 97 }, { 120, 89 }, { 22, 54 }, { 22, 136 }, { 136, 134 }, { 77, 121 }, { 58, 21 }, { 69, 93 }, { 25, 83 }, { 191, 50 }, { 28, 64 }, { 184, 110 }, { 129, 17 }, { 193, 118 }, { 184, 132 }, { 79, 160 }, { 104, 71 }, { 18, 79 }, { 180, 20 }, { 4, 148 }, { 138, 89 }, { 176, 135 }, { 16, 159 }, { 92, 116 }, { 11, 162 }, { 188, 60 }, { 5, 23 }, { 92, 64 }, { 162, 130 }, { 26, 128 }, { 48, 107 }, { 174, 90 }, { 192, 163 }, { 127, 54 }, { 188, 43 }, { 160, 100 }, { 171, 111 }, { 101, 101 }, { 198, 33 }, { 31, 74 }, { 26, 135 }, { 157, 139 }, { 18, 108 }, { 14, 132 }, { 56, 45 }, { 167, 6 }, { 124, 71 }, { 181, 115 }, { 44, 102 }, { 138, 91 }, { 39, 63 }, { 44, 94 }, { 22, 47 }, { 98, 97 }, { 7, 97 }, { 52, 139 }, { 72, 149 }, { 134, 39 }, { 21, 124 }, { 24, 156 }, { 126, 28 }, { 197, 31 }, { 63, 68 }, { 118, 98 }, { 181, 41 }, { 163, 142 }, { 138, 68 }, { 133, 32 }, { 198, 85 }, { 187, 22 }, { 2, 81 }, { 190, 125 }, { 17, 7 }, { 19, 2 }, { 174, 117 }, { 61, 9 }, { 188, 121 }, { 77, 77 }, { 158, 112 }, { 175, 31 }, { 116, 24 }, { 98, 139 }, { 6, 154 }, { 19, 85 }, { 176, 51 }, { 110, 106 }, { 195, 42 }, { 147, 114 }, { 123, 3 }, { 9, 46 }, { 162, 104 }, { 42, 154 }, { 64, 129 }, { 122, 125 }, { 13, 107 }, { 20, 134 }, { 85, 25 }, { 195, 139 }, { 119, 127 }, { 27, 89 }, { 74, 84 }, { 90, 130 }, { 109, 144 }, { 113, 108 }, { 51, 102 }, { 100, 36 }, { 97, 109 }, { 85, 66 }, { 168, 74 }, { 71, 36 }, { 33, 26 }, { 69, 38 }, { 83, 9 }, { 104, 39 }, { 103, 31 }, { 0, 126 }, { 48, 33 }, { 76, 14 }, { 122, 135 }, { 135, 149 }, { 54, 35 }, { 34, 150 }, { 23, 104 }, { 56, 114 }, { 132, 92 }, { 49, 32 }, { 18, 44 }, { 182, 17 }, { 79, 3 }, { 124, 24 }, { 28, 0 }, { 6, 43 }, { 190, 92 }, { 68, 44 }, { 97, 148 }, { 165, 151 }, { 54, 109 }, { 120, 32 }, { 73, 127 }, { 191, 62 }, { 91, 104 }, { 6, 108 }, { 30, 131 }, { 173, 67 }, { 134, 132 }, { 3, 51 }, { 21, 99 }, { 121, 0 }, { 156, 7 }, { 196, 112 }, { 124, 22 }, { 140, 99 }, { 60, 64 }, { 149, 43 }, { 168, 101 }, { 160, 38 }, { 57, 110 }, { 96, 19 }, { 162, 141 }, { 129, 114 }, { 21, 131 }, { 195, 4 }, { 170, 112 }, { 88, 58 }, { 98, 6 }, { 61, 92 }, { 0, 30 }, { 158, 147 }, { 140, 161 }, { 123, 15 }, { 37, 21 }, { 32, 116 }, { 84, 59 }, { 5, 18 }, { 180, 119 }, { 151, 68 }, { 145, 142 }, { 180, 135 }, { 121, 4 }, { 84, 109 }, { 140, 136 }, { 58, 49 }, { 78, 64 }, { 21, 54 }, { 139, 74 } };
//					{ { 244.45824720006732, 150.87657444379155 }, { 225.58160392126234, 168.16912597688656 }, { 246.78048479615813, 124.41587479591739 }, { 261.18496609803844, 172.69361726501938 }, { 216.50249703903356, 145.93159586882481 }, { 269.14718758251337, 135.17149149575036 }, { 236.64240212725508, 179.95112853743254 }, { 230.2025153519796, 123.42803025162057 }, { 274.2643161105507, 161.7617030050238 }, { 214.40443054581132, 163.2823530699967 }, { 258.56277587195973, 120.73601592575665 }, { 254.6411803957443, 183.34137007461953 }, { 214.387614243599, 133.45002799262932 }, { 279.1021381739946, 143.2602121969266 }, { 223.65400230500344, 180.46847198101915 }, { 239.72110900788675, 114.32040595998096 }, { 273.161614424572, 175.06780272809252 }, { 206.28862484369353, 152.4550081634814 }, { 272.0000046515412, 123.46884719789269 }, { 242.63379627083447, 190.3320078738277 }, { 218.7467626659748, 120.06562289934921 }, { 284.8464566707154, 156.31075090484356 }, { 210.5032022533699, 174.50161031203447 }, { 253.66978378483094, 109.9303707798183 }, { 265.6209157041523, 187.80825276948946 }, { 203.34673941252123, 137.76088215511413 }, { 284.15845612489187, 132.53407096383572 }, { 227.35379380649533, 191.7467914071334 }, { 229.2718585918462, 108.65454541059071 }, { 284.6708105558627, 172.01117074035298 }, { 199.99736315966874, 162.5963207900634 }, { 269.6204564172531, 111.7435578021169 }, { 252.42962791550312, 197.25975759819875 }, { 206.82789767207726, 121.73351665948178 }, { 292.4165442444719, 146.9033275651733 }, { 211.4252823984405, 186.58424147295295 }, { 244.69806048817804, 101.71773689323484 }, { 277.83064583568864, 187.66477314841788 }, { 194.49733190581077, 146.24620804379452 }, { 284.82411289738286, 120.24032152980948 }, { 235.29953688225388, 201.22094603781744 }, { 216.9431127255001, 107.15227353310016 }, { 294.75122878903903, 164.6597956148999 }, { 197.63479299342984, 174.9055980694521 }, { 262.9189684858532, 101.08194852244829 }, { 264.66006660488733, 200.5018409365575 }, { 195.61681844977676, 127.72970988726658 }, { 296.5522798368731, 134.8154274086595 }, { 216.66171148822454, 198.30643471545156 }, { 232.8201160030224, 96.67982715651291 }, { 290.0190495105368, 183.24083428170923 }, { 188.55481707065144, 157.84371127666026 }, { 281.2628656629397, 107.63699850627636 }, { 246.54372670950323, 208.06400178107046 }, { 203.98231675876877, 109.80390598328017 }, { 302.48393342836164, 153.84346679081733 }, { 199.33388718545953, 188.1605678054235 }, { 252.60598796295113, 92.47757405706781 }, { 278.1404288615186, 199.79352433766573 }, { 186.16602440148165, 137.46367019216189 }, { 296.86765352513993, 121.18349996640582 }, { 225.7415636716589, 208.57101927578577 }, { 219.11499895413132, 95.31377262570992 }, { 301.05738640292896, 174.88881811436022 }, { 186.1175568077005, 171.53964979256514 }, { 273.7102765270859, 95.8723090587708 }, { 260.1447349556699, 211.58604022279954 }, { 191.54591094582764, 116.48838499839474 }, { 307.0969445257018, 140.42586697978234 }, { 205.0852393172723, 201.20701953901025 }, { 239.4623013595531, 86.77513393139604 }, { 291.72849964719063, 195.03591063762696 }, { 179.3837280058508, 150.2416486971989 }, { 293.1592720499649, 107.12864182979817 }, { 238.06151585378063, 216.41535136035952 }, { 204.67416718124753, 97.92322530823436 }, { 309.93754210710404, 163.11894344004523 }, { 187.58482120713322, 186.28015037525225 }, { 262.5814565288467, 85.9909998226237 }, { 275.09366364814883, 211.29719258528323 }, { 180.70656418392193, 126.88706354281793 }, { 308.0151690895183, 125.3640715031368 }, { 214.66739842503478, 212.95293348132492 }, { 224.38685067361257, 84.62923927108292 }, { 304.321327330049, 186.3531271831838 }, { 175.99804469303203, 165.22861287048823 }, { 285.4543986573408, 93.75648652394304 }, { 252.85568558790692, 221.04410816198458 }, { 190.59787463673277, 104.57675568761246 }, { 315.80354072109236, 148.6232408177506 }, { 193.11942896901058, 200.97819004889263 }, { 248.49042617768487, 78.90291519291969 }, { 290.32433874660916, 206.94251676929457 }, { 172.4210028930813, 140.46753998529877 }, { 304.89418927977204, 109.69604576685613 }, { 227.63263526716756, 222.4018595896577 }, { 208.38231712840843, 86.47063065310384 }, { 314.8900371604149, 174.10573840150124 }, { 176.5223982805805, 181.46393045033471 }, { 274.02450673022975, 82.12092694836126 }, { 269.2119165734474, 221.86528814421564 }, { 177.95755241845868, 115.09333740819427 }, { 317.98935225894127, 132.2593284380855 }, { 202.63158385609484, 214.55231735196926 }, { 232.23459506654254, 75.34314481924157 }, { 304.75280750986565, 198.52036962461972 }, { 167.48801436401465, 156.4978659217346 }, { 297.64128605554873, 94.50076104718632 }, { 243.3192719693305, 228.69703981132147 }, { 192.51528248170206, 92.48217002457038 }, { 322.52596156802196, 158.88080972124447 }, { 181.22829685077147, 197.9007036574262 }, { 259.37819746922025, 73.17617715815862 }, { 286.11013665257195, 218.52078935429836 }, { 167.74633084720864, 129.0461741799969 }, { 316.05287763495835, 115.01350331042792 }, { 215.77910511043973, 225.9773465694629 }, { 214.75976676289858, 75.8346575630923 }, { 317.3288820089281, 186.28605252436012 }, { 166.5077960162961, 174.07917961790827 }, { 286.4231785422372, 80.84620199983962 }, { 260.881529174571, 231.16518354013607 }, { 177.86421180501623, 102.5868508604379 }, { 326.48732361898533, 141.46524236852045 }, { 190.12899569752648, 213.4573713749939 }, { 242.23807866640038, 67.72744767539571 }, { 302.473097271355, 210.90713214737752 }, { 160.82738391156676, 145.78195240098572 }, { 309.8010393373444, 97.95131321833037 }, { 231.98252727239114, 234.33799643808072 }, { 197.11246566749213, 80.65891432571863 }, { 327.0913819493279, 170.74110578252257 }, { 169.84839414714833, 192.19119159678164 }, { 271.65990962545925, 69.73292424524276 }, { 279.33240732759884, 229.35403258262886 }, { 165.46223587762915, 116.44917011988923 }, { 326.2405906377512, 122.80589269056648 }, { 202.97602189332875, 227.07526307195207 }, { 223.50425644815505, 66.38605952490214 }, { 317.2238155618253, 199.18384041163714 }, { 157.8858049272856, 164.4536332599693 }, { 299.3030805338262, 82.1606789510314 }, { 250.45386360561957, 238.8799249804007 }, { 180.38407560059022, 89.837684670785 }, { 333.221722040857, 152.60885251156935 }, { 177.6227146073876, 209.7465826025709 }, { 254.0049394829037, 62.03829857166204 }, { 297.59618331826863, 223.06003470554614 }, { 156.23963323856555, 133.49028075639953 }, { 321.4926684854447, 103.95940144681722 }, { 219.26940817320883, 237.77745447708423 }, { 204.2068980769758, 69.53293257143658 }, { 329.3453292002351, 183.76825022993387 }, { 159.38801851135963, 184.06476940650813 }, { 284.89049868167785, 68.69168889717945 }, { 270.23743903329944, 239.05411761605012 }, { 165.6506385847187, 103.12794779496024 }, { 335.0917912955628, 132.79738583463654 }, { 189.6782598860465, 225.65074373292683 }, { 234.31225987175003, 58.46547913903483 }, { 314.56723434357355, 212.3437271576455 }, { 150.97001136876486, 152.91647532536615 }, { 312.21132123988735, 86.03473591316963 }, { 238.28153342054554, 244.72498430577144 }, { 185.45058056269687, 77.29325441408768 }, { 337.93890157174957, 165.3165177021455 }, { 165.5517018902292, 203.5229219828556 }, { 267.14306949805245, 58.49683188962534 }, { 290.260661040431, 234.5515119507993 }, { 153.91196566068453, 120.03072312779632 }, { 332.3028864975796, 112.35183658942378 }, { 205.60072446244098, 238.86333769527693 }, { 213.59207058396046, 59.50043270193463 }, { 329.1718148392496, 197.53142631985787 }, { 150.22360173455326, 173.75975579889433 }, { 298.6321552437007, 70.13165654773842 }, { 259.0936215604607, 247.26696093139762 }, { 168.3534595203343, 89.52423289619418 }, { 342.27717096842275, 144.69147798591948 }, { 176.3283449702712, 221.69933517609854 }, { 246.86131861858283, 52.375188603106125 }, { 309.39209089670493, 225.32551969401263 }, { 146.0335729909321, 139.8137655982925 }, { 324.7131670912539, 92.39834113233042 }, { 224.73167240275382, 248.4581959815951 }, { 192.95720765133805, 65.37973717366926 }, { 340.4293502057572, 179.20271514789354 }, { 154.3305342053701, 194.92854107050272 }, { 281.25138064320424, 57.2784748640544 }, { 280.6432013887789, 244.9815554166752 }, { 153.98453485992755, 105.81699052811001 }, { 341.84912200147016, 122.91879316684569 }, { 191.39977041132224, 237.4917018297796 }, { 225.02661114879118, 50.92380104658733 }, { 326.50377844898344, 211.60087138684727 }, { 142.69473939366338, 161.5472289442421 }, { 312.45181536682594, 74.08259729891293 }, { 246.19967886673012, 253.67820086518196 }, { 173.56304440383087, 76.07119477493737 }, { 347.5098633573829, 158.1640652934813 }, { 163.35635114295448, 215.26597539850673 }, { 260.8044043214306, 48.372496962343405 }, { 301.7801913473091, 237.70542097554656 }, { 143.3023956498776, 125.51282975138412 }, { 336.3942363879421, 101.13350889582938 }, { 210.1891235791297, 249.88667252990322 }, { 202.75074895233894, 54.49902794834773 }, { 340.5345702182356, 193.8679967695994 }, { 144.3456389655683, 184.15064998777336 }, { 295.91884291330814, 58.50621913461066 }, { 268.96354240636015, 253.98207284706012 }, { 156.54397439750971, 91.26858343130361 }, { 349.78435008679804, 135.40972346072027 }, { 177.0913404606772, 233.61285177554305 }, { 238.2311122995718, 44.12641947339492 }, { 321.32876694790343, 225.54971467736408 }, { 137.09877109338197, 147.7332886068823 }, { 325.9237664789548, 80.51969851432189 }, { 231.8860338944541, 258.018744130429 }, { 181.2175719579795, 63.19016102649192 }, { 350.55101313527234, 172.8629548900916 }, { 151.1760264970178, 206.44895774992824 }, { 275.7702725705109, 46.664258012679994 }, { 291.86548851435083, 249.08040025951055 }, { 142.94981005078836, 110.40110058389948 }, { 346.86527285660077, 112.07170994208246 }, { 195.05452176493213, 248.8718737757337 }, { 214.62944448131844, 45.02365565340192 }, { 338.1518151106303, 208.90161275455117 }, { 135.94999183514284, 171.4226408222303 }, { 310.7277557369424, 62.246286822586384 }, { 255.48486855028, 261.2223188792216 }, { 161.6195298223085, 76.80693580042595 }, { 355.8025631483768, 149.53371780123203 }, { 163.09735604034034, 227.23468833071547 }, { 252.88921824051857, 39.387239460205265 }, { 313.69171310283133, 238.95879692053455 }, { 133.68548097403482, 132.6572628704737 }, { 338.6348398468129, 89.36139352020578 }, { 216.51233715873315, 260.069861465699 }, { 191.19952067351593, 51.28512755794212 }, { 351.2145949811358, 188.4109795599594 }, { 140.17908245689273, 195.40080335638766 }, { 291.36717965431546, 47.402395544628575 }, { 279.83427879282925, 259.07404758102405 }, { 145.09250340161248, 94.88184611482794 }, { 355.76806692668526, 124.9943569597743 }, { 179.73951852276724, 245.33321255901484 }, { 228.34414920488925, 37.29087106484964 }, { 333.2371826932286, 223.88673319349664 }, { 129.45728837375367, 157.02224741284743 }, { 325.259355798834, 68.5055935714428 }, { 240.51131280306575, 266.41456738896056 }, { 169.18111461135362, 62.849484716230464 }, { 359.6442176702295, 164.9626377494275 }, { 149.83067885830968, 218.42402945978108 }, { 268.6513644518502, 36.93562710047348 }, { 303.6956110877026, 251.42304525376568 }, { 132.6522968924346, 116.68739708375051 }, { 350.1909043334068, 100.4693476170857 }, { 200.46262484436028, 259.66757621941656 }, { 203.33632521603982, 40.73621598743813 }, { 349.37133663473617, 204.41132472105548 }, { 130.72867228482005, 182.32693247997963 }, { 307.18864419607456, 50.68049395492828 }, { 265.9232063219296, 267.343004858985 }, { 149.78766632180879, 79.36802783817281 }, { 362.781728451739, 139.6354558486564 }, { 164.66031841899627, 239.2503147007182 }, { 243.60162145484725, 31.596598602166466 }, { 325.8072511481887, 238.40717272595987 }, { 125.1361839960139, 141.267644314776 }, { 339.10073909194404, 77.23076091968962 }, { 224.38347350400133, 269.31952266831166 }, { 179.1389891234426, 49.80261621113921 }, { 361.1012393507008, 181.3361453008351 }, { 137.68799110588265, 207.3062530920368 }, { 285.1403116034056, 36.946817251095254 }, { 291.50047588118355, 262.55859157580335 }, { 134.1402526580123, 100.2148613671324 }, { 360.22391625675397, 113.6501807380433 }, { 184.1389182135316, 256.7061500095643 }, { 217.40277336587206, 31.892758718117392 }, { 344.95161358664916, 220.4542827691718 }, { 123.15267632731017, 167.482607381285 }, { 322.8204975465184, 56.53152793572917 }, { 250.41556587334628, 273.5834170037823 }, { 157.0313746070173, 64.27501480827044 }, { 367.6288130125826, 155.68591665987756 }, { 150.23022038766823, 230.66396338860994 }, { 260.0694158940863, 28.189693149851664 }, { 315.9393338721162, 252.05495738683024 }, { 123.20498890358624, 124.51200896053868 }, { 351.8524703258878, 88.30856050727337 }, { 207.47247379057794, 269.75113519615104 }, { 191.34490711666876, 38.05407950834598 }, { 360.0213026178173, 198.26809939752104 }, { 127.04229953692135, 194.06345127860706 }, { 301.9579470632068, 39.53821260942388 }, { 277.3208072847731, 272.01011302453537 }, { 138.2309030670812, 83.64659880161562 }, { 368.3990399763961, 128.6586782796688 }, { 167.95377944154046, 251.14051964798398 }, { 233.12487045188757, 25.06646098521429 }, { 337.94721142090157, 236.12492889612952 }, { 117.73720719770344, 151.1684412900653 }, { 337.84872833198835, 64.92679549354597 }, { 233.63621775386076, 277.53702585124444 }, { 166.75823246125157, 50.012634969642136 }, { 370.08096046170925, 172.79919499964254 }, { 136.85163067794457, 219.67573692701828 }, { 277.38203855716284, 27.26680770726327 }, { 303.77039551829347, 264.4382837290399 }, { 123.8270886053851, 107.13690019337453 }, { 363.1364477961805, 101.56771629702698 }, { 190.17292505624957, 267.5805860917313 }, { 205.5946266886029, 27.965282181181593 }, { 356.311182774215, 215.3539524900514 }, { 118.2354553294397, 178.93122609743864 }, { 318.69298318117023, 44.766695973455555 }, { 261.42171412470645, 279.45802438366565 }, { 144.94432720983528, 67.39738647491325 }, { 374.42149171050033, 145.2023114426238 }, { 152.32219528830618, 242.98758182333518 }, { 250.18497159016576, 20.524963806153835 }, { 328.41188337987546, 251.01134978021247 }, { 114.72063450536186, 133.72469212724454 }, { 351.8677573167942, 75.77609662661415 }, { 215.94535024960095, 278.9966322668696 }, { 178.84229047737387, 36.97692094183802 }, { 369.9637965262192, 190.5980135489278 }, { 124.90789618185289, 206.44635448573362 }, { 295.14810085548055, 28.96809517427758 }, { 289.4946705811604, 275.1895315329175 }, { 127.1072871737859, 89.54472699915759 }, { 372.6036967341687, 116.78199804370372 }, { 172.89507693547324, 262.73926942973634 }, { 221.6316948009621, 19.863779996886535 }, { 349.9390363255297, 232.1788579667692 }, { 111.57098886951118, 162.19424079245357 }, { 334.92925848293334, 52.627073825342805 }, { 244.11383328429622, 284.62523920834207 }, { 154.23831207171176, 51.881405972295454 }, { 378.0428405036831, 162.9469128230031 }, { 137.65308599051775, 232.32625933460696 }, { 268.2280094607638, 18.48605569319858 }, { 316.46066233519156, 264.71312885070694 }, { 114.28835779394086, 115.52452959768515 }, { 364.4888492047746, 88.92917362218455 }, { 197.73129288427953, 277.81071781488174 }, { 193.09923012318407, 25.54183892912289 }, { 367.1609096604582, 208.68194751280214 }, { 114.75524038609188, 191.19276153333374 }, { 312.95824521688394, 33.373910668581374 }, { 273.36003878139275, 283.9730559373826 }, { 133.08921772902892, 72.15054411603828 }, { 379.941826629257, 133.6745301530833 }, { 156.05639803264037, 255.22065019094686 }, { 239.1524090059834, 14.036214801332903 }, { 340.93550976573147, 248.3262385162948 }, { 107.30750025578001, 144.1806418859678 }, { 350.254387942185, 63.05172836283133 }, { 225.74717740375553, 287.28327274050724 }, { 166.0087709802355, 37.50357923008234 }, { 379.06655449420396, 181.52335666110963 }, { 124.34069558529433, 219.29631673837736 }, { 286.8686516908348, 19.112728801256566 }, { 302.2679171531577, 276.84983338914844 }, { 116.56850619525738, 96.96645543707075 }, { 375.3484098464495, 104.17840127675927 }, { 179.40257257848822, 273.88706552043703 }, { 209.28978187031026, 16.051247399939 }, { 361.6167614571606, 226.6356712086819 }, { 106.71504207384075, 174.1840083185612 }, { 330.39363050898544, 40.50263666670229 }, { 255.66323033258166, 290.492526952713 }, { 141.7539225564612, 55.3733937548628 }, { 384.8819300631482, 151.92322402991527 }, { 140.07281230842798, 245.08091568693146 }, { 257.81184665840067, 10.72191191162375 }, { 329.3936151219427, 263.38658654540575 }, { 105.65280467385895, 125.25547868797128 }, { 364.26853120948846, 75.91151912639411 }, { 206.70405317612676, 287.2575010098319 }, { 180.0914175455257, 24.650901404240713 }, { 377.3526215450359, 200.53508760396753 }, { 112.75609415143319, 204.09592507618078 }, { 305.6992188546325, 22.509234764818387 }, { 286.0641731172133, 287.06927581700955 }, { 121.62870347495274, 78.46605720837015 }, { 384.1159456689157, 121.26268783024312 }, { 161.37940301059803, 267.1952584008599 }, { 227.12431878749095, 8.811302668046011 }, { 353.33842729285766, 244.03766714021384 }, { 101.0666663320848, 155.7359978467136 }, { 347.0347718336665, 50.30971577561296 }, { 236.7439549855336, 294.49761583337835 }, { 153.01966876175914, 39.6274039780464 }, { 387.20473294904554, 171.16722350200013 }, { 125.34980516238672, 232.438373704286 }, { 277.2308357223634, 10.10731541474837 }, { 315.4673447673805, 276.9662807751497 }, { 106.75899812988808, 105.81326043313014 }, { 376.59306273266066, 91.01794247465283 }, { 187.39120358157012, 284.4314246377535 }, { 196.26468426880106, 13.683985369687633 }, { 372.8211077517892, 219.56633283912342 }, { 103.2390035219122, 186.978005290777 }, { 324.2981875321094, 28.71826084255187 }, { 268.13174437205635, 295.05534225521495 }, { 129.47400246815366, 60.4472801742905 }, { 390.5014845837909, 139.87252084937893 }, { 144.08479917292485, 257.7679690138576 }, { 246.2685427140279, 4.083504475961263 }, { 342.3960981975496, 260.46919963923295 }, { 98.04103191369148, 136.20520255528456 }, { 362.4705324261938, 62.68794203992003 }, { 216.97806946167844, 295.78985464392673 }, { 166.7430320362674, 25.31284875724468 }, { 386.7458341362342, 191.01423432080156 }, { 112.27360538171459, 217.47158322082834 }, { 297.0037249034681, 12.321410633416235 }, { 299.3690297346347, 288.69621329732877 }, { 110.7185789769712, 86.26980537721373 }, { 386.8789400906916, 108.12652647644555 }, { 168.23133916374073, 278.7498166779992 }, { 214.25384550208173, 4.929017926011909 }, { 365.45454875198476, 238.19082308577467 }, { 96.09014141507416, 168.24540808782908 }, { 342.2390959255716, 37.71931515116465 }, { 248.79925301266618, 300.5352000148922 }, { 140.04606081604194, 43.333406257837964 }, { 394.262278059982, 159.65606726652013 }, { 127.93551113144095, 245.70098542953605 }, { 266.3501568622065, 2.078552514244194 }, { 328.9223069875969, 275.523355615998 }, { 97.81507523368793, 115.98147027264284 }, { 376.30706831008746, 77.46902993369258 }, { 196.7699172632196, 294.2275205799427 }, { 182.72124982513293, 12.807369379045923 }, { 383.3998988753733, 211.0485914720179 }, { 101.20264138244187, 200.41623416643773 }, { 316.7067957728779, 17.432255341912736 }, { 281.3654755419948, 298.24003145458437 }, { 117.56173201734491, 67.0535489615922 }, { 394.8145792120034, 126.94141002443112 }, { 149.6542338731645, 270.2206632722989 }, { 233.73627686846712, -1.329686747317652 }, { 355.2990988155782, 255.9808399196149 }, { 91.56426231335658, 148.2450142169552 }, { 359.09966397711537, 49.4283792942 }, { 228.43513414299528, 303.2857142717423 }, { 153.22359921123572, 27.537932180521437 }, { 395.2086244908228, 180.22621374526224 }, { 113.33297158440718, 231.15194976695088 }, { 286.9663974715291, 2.951162654680928 }, { 313.10987226805327, 288.81395685638245 }, { 100.50724755979576, 95.48005933008562 }, { 388.1765307222032, 94.4264499829448 }, { 176.5439920757842, 289.7294217297116 }, { 200.69581554832763, 2.45755007651951 }, { 377.12369699806914, 230.8399029660772 }, { 92.4594610026027, 181.5608215898525 }, { 335.90714674423634, 25.444713900412694 }, { 261.77293749081093, 305.3018103805522 }, { 127.25485367876297, 48.596488625519584 }, { 400.1330588434483, 147.1210281571028 }, { 132.0875690756218, 258.9158348616371 }, { 254.3477577386613, -4.856366661754208 }, { 342.46439213762824, 272.51640127958626 }, { 89.86405559090306, 127.36086318663669 }, { 374.4709367768469, 63.69885697289132 }, { 207.44024863699124, 303.1389238849466 }, { 168.82415356808144, 13.455539531113232 }, { 393.2086754335429, 201.16841291797752 }, { 100.65444999243203, 214.33781509564892 }, { 307.6922784294977, 6.7959686405401385 }, { 295.20858323336364, 299.9841525936357 }, { 106.17415750062295, 75.1330593632715 }, { 397.74533804023656, 113.27949118216863 }, { 156.73609049235853, 282.2774866657626 }, { 220.35725748186448, -5.428968327921524 }, { 367.93790343080815, 249.9520965135692 }, { 86.32329271081463, 161.2411867007447 }, { 354.17186882709484, 36.299460838374344 }, { 240.95102337637366, 309.63298564938674 }, { 139.70036845253196, 31.324950391275777 }, { 402.61849236310513, 168.28480089618478 }, { 115.9477144303925, 244.97045447455963 }, { 275.6897001562326, -5.469572614558302 }, { 327.12209979859443, 287.39439436606835 }, { 91.13504180511242, 106.00643999208987 }, { 387.96625989357415, 80.32381806697197 }, { 186.23974511033975, 299.98644762588094 }, { 186.6071035894787, 1.4533472013722246 }, { 388.1921091634073, 222.0491831911472 }, { 90.24460128439412, 195.53105508392917 }, { 328.08938697508347, 13.644610722026755 }, { 275.5206762074221, 308.7145082610655 }, { 114.80844542869025, 55.38036797778773 }, { 404.7218384759668, 133.6984831274856 }, { 137.78413117243113, 271.91808674452125 }, { 241.351019633478, -10.590664791018185 }, { 355.92760597939116, 267.9526864365887 }, { 83.02341210042121, 139.83402237201665 }, { 371.07732641820974, 49.87329247649798 }, { 219.2956353317441, 311.03839155452647 }, { 154.73785842096254, 15.650366574748716 }, { 402.11142305481087, 190.02070063966985 }, { 101.63063611046559, 228.58095818261086 }, { 297.3371670081563, -3.0468776476985795 }, { 309.5031944239911, 300.237465840435 }, { 95.46158655901158, 84.61626821206022 }, { 399.22989841698814, 99.03950932063617 }, { 165.2743294341222, 293.7827150934661 }, { 206.2779322723569, -8.137368960311761 }, { 380.15258032581363, 242.42509506619132 }, { 82.4075891768048, 175.05468807531776 }, { 347.7150560326854, 23.46408723854016 }, { 254.39518033213832, 314.7304166159591 }, { 126.3379500356504, 36.66040375297615 }, { 408.8631973171645, 155.31108534812296 }, { 120.11882550839454, 258.7620471861028 }, { 263.284337721513, -12.818447108405422 }, { 341.24149306289246, 284.422072604952 }, { 82.73346085846723, 117.7494635184764 }, { 386.2183532632032, 65.98075709369381 }, { 197.23108379231658, 309.3812691121689 }, { 172.14649949023467, 1.9602546508666592 }, { 398.51311928959353, 211.89355360212056 }, { 89.50312106104275, 210.00187094113465 }, { 318.84752455222093, 2.4715748665690853 }, { 289.89396240801426, 310.7024834465077 }, { 102.86412799113626, 63.63697352183139 }, { 407.94511810975416, 119.5300754406023 }, { 144.99111370431314, 284.54694119197245 }, { 227.49364427584743, -15.028813380605357 }, { 369.14888012940435, 261.8520655484773 }, { 77.39995291677843, 153.27620298253996 }, { 366.1317296395351, 36.156418624710014 }, { 232.2212344179611, 317.80867083697854 }, { 140.62620002992435, 19.4007427876179 }, { 409.9813579923963, 177.70952800193243 }, { 104.15438919794977, 242.9833306486278 }, { 285.7339815131203, -11.960953795409381 }, { 324.08972196310253, 298.9626812915891 }, { 85.5668461257319, 95.42290299344182 }, { 399.21717426931843, 84.37708622641031 }, { 175.20152501549308, 304.5871288318216 }, { 191.64877020599297, -9.390415680210594 }, { 391.78866851805583, 233.45391426406619 }, { 79.89450059998072, 189.54134838081683 }, { 339.76955992332574, 11.08077095318626 }, { 268.6308303132983, 318.4883909319622 }, { 113.29768899665365, 43.51799401115818 }, { 413.84155712803334, 141.43340846873326 }, { 125.83422668277527, 272.3637889633355 }, { 249.86924613200276, -18.983501264418493 }, { 355.30476820346456, 279.894774429613 }, { 75.42437184967218, 130.60049814258386 }, { 382.91633251292916, 51.55964607803796 }, { 209.42049935380354, 317.7830556197831 }, { 157.47423639813113, 4.008868997896002 }, { 407.9479445408167, 200.4586672569213 }, { 90.27948469795143, 224.81640473817896 }, { 308.2547156324742, -7.9287289695034815 }, { 304.74049580724153, 311.20775969667085 }, { 91.57332652599379, 73.30619094675195 }, { 409.7318681204519, 104.76238232288031 }, { 153.6618889173605, 296.6463763538911 }, { 212.9153721610971, -18.087286830682075 }, { 381.9687932763752, 254.24733855505275 }, { 73.08904768675336, 167.55556128034246 }, { 359.652883193452, 22.709834662887317 }, { 246.09409732051122, 323.34328938025135 }, { 126.65171907463909, 24.70212247005867 }, { 416.701730168477, 164.34801912711401 }, { 108.23537416107035, 257.38269242954533 }, { 272.9851683964411, -19.820134217306787 }, { 338.80746455366994, 296.13601217863993 }, { 76.62446480965968, 107.46196570165054 }, { 397.6694531696378, 69.45015957866566 }, { 186.43881027777184, 314.54883150405215 }, { 176.62374308037886, -9.136772414107725 }, { 402.6979939336241, 223.10470064950954 }, { 78.84858132466258, 204.55233227818803 }, { 330.3883148289816, -0.6971727736524542 }, { 283.5154044846422, 320.82964149233453 }, { 100.73686746358948, 51.858389674803206 }, { 417.4641935949445, 126.78699159969398 }, { 133.06847593437868, 285.61563387019186 }, { 235.5712754408375, -23.86393890442693 }, { 369.1503381689506, 273.82387178073395 }, { 69.31920974944242, 144.44202400664622 }, { 378.0574261515883, 37.22238018473878 }, { 222.70068896042355, 325.0705926069536 }, { 142.75128365162908, 7.616070852278796 }, { 416.3665234357794, 187.84080185165425 }, { 92.60454034590327, 239.81583847564502 }, { 296.39548940082, -17.41839123462495 }, { 319.9048175435112, 310.18576894888014 }, { 81.08074121115467, 84.31587283058504 }, { 410.02415220021805, 89.54632255299393 }, { 163.73722959408508, 308.06600934946346 }, { 197.76143681715328, -19.695233494703444 }, { 394.23242977375196, 245.184372803935 }, { 70.1739128091831, 182.53364973831668 }, { 351.6729568814404, 9.691805824087709 }, { 260.7836105527353, 327.5473100058904 }, { 112.97482296457946, 31.536266616541397 }, { 422.1666139898094, 150.05796601509147 }, { 113.8694099692018, 271.6177163980816 }, { 259.2027787323662, -26.508320006478158 }, { 353.49540357369744, 291.7475612894098 }, { 68.75982326799263, 120.63199072912889 }, { 394.5628463369277, 54.41821482209812 }, { 198.89606840816026, 323.53412188429957 }, { 161.35959131882217, -7.338751650282916 }, { 412.7395582162268, 211.45554581878076 }, { 79.32102017016174, 219.9348342134285 }, { 319.63680264320993, -11.722526327020148 }, { 298.9011910225292, 321.6898778105149 }, { 88.8077982296648, 61.629206775271626 }, { 419.654213284275, 111.5133337056863 }, { 141.78267558210717, 298.36133566005003 }, { 220.52464079252388, -27.37085756539952 }, { 382.61921379001194, 266.23449038563393 }, { 64.51820153226001, 159.14812451908705 }, { 371.65280704041436, 23.129481215237078 }, { 236.95498545087267, 331.13309842960547 }, { 128.13847481864804, 12.784715113491302 }, { 423.64836891152936, 174.14649523132115 }, { 96.49513908061351, 254.8402490052785 }, { 283.36545208291284, -25.86814159819943 }, { 335.22912914064045, 307.6058016713149 }, { 71.52343764499406, 96.58206328865367 }, { 408.7776450721814, 74.0363728006864 }, { 175.14545997031033, 318.6620256193579 }, { 182.18182213622575, -19.795058116230052 }, { 405.79032274095533, 234.72202778468713 }, { 68.72496741291732, 198.06611321142302 }, { 342.23755536114606, -2.743687316541042 }, { 276.1521401201583, 330.33803269376347 }, { 99.75283401028474, 39.871163131462495 }, { 426.2816642043705, 134.96924089979427 }, { 121.03831152591292, 285.52893490247664 }, { 244.50794200566307, -31.920244125349313 }, { 367.9931377861127, 285.80155726052936 }, { 62.08830463991333, 134.82150533732985 }, { 389.88760235323366, 39.44136884192923 }, { 212.47232356243074, 331.41838248842714 }, { 146.0149325547331, -3.97270427655252 }, { 421.78046063275957, 198.5961687161364 }, { 81.3491763380051, 235.53293848073494 }, { 307.5928112298331, -21.855118909577698 }, { 314.6361575140796, 321.01832194067794 }, { 77.65685459565307, 72.765171854649 }, { 420.3478123050759, 95.75943276916401 }, { 151.92455470482594, 310.4494323493243 }, { 204.870193510769, -29.42791861484511 }, { 395.5559960223832, 257.1655093097975 }, { 61.10963475390875, 174.5851599538382 }, { 363.7276747509892, 9.439104587202934 }, { 252.05797032988406, 335.8710125410452 }, { 113.79552088623277, 19.503465562571847 }, { 429.6834083001769, 159.49199792527847 }, { 101.95388680082249, 269.7295815986043 }, { 269.2708095166965, -33.1582352646912 }, { 350.5542458118802, 303.4513366961627 }, { 63.02992003917109, 110.00940131266424 }, { 405.9620424846156, 58.389641548083915 }, { 187.80278142367678, 328.298140015788 }, { 166.33037036632692, -18.342909809624558 }, { 416.49944290217945, 222.9319101264792 }, { 68.79927016012195, 214.003540199578 }, { 331.40555630988825, -14.447958616583861 }, { 292.0558355848933, 331.645629641313 }, { 87.138965755691, 49.66110360221347 }, { 428.964818904713, 119.21904556169798 }, { 129.70988008505503, 298.9597701039962 }, { 229.03017465443955, -35.96223789231968 }, { 382.1419129959737, 278.3164511904365 }, { 56.71446955263508, 149.90965640371724 }, { 383.648290843673, 24.679348409839164 }, { 227.05629784172783, 338.0869576158529 }, { 130.74925562057493, 0.9707102768518325 }, { 429.6968232559797, 184.62743302390055 }, { 84.95622405527666, 251.1886022901141 }, { 294.34603027282924, -30.96279255033801 }, { 330.56490377724305, 318.77814715629484 }, { 67.42347021407281, 85.1884441036956 }, { 419.49479578213857, 79.67687036884416 }, { 163.42870628846362, 321.73427352413177 }, { 188.75455001724822, -29.97194436782962 }, { 407.8099231408361, 246.6694111767668 }, { 59.16918654680529, 190.6125875140637 }, { 354.3211949012624, -3.694020268489652 }, { 267.8762364554084, 339.19673604461354 }, { 99.87994588651404, 27.746765307638583 }, { 434.37278853369867, 144.00257382566332 }, { 108.96902401472781, 284.3247113508333 }, { 254.22773676612317, -39.17931741044336 }, { 365.7206471075466, 297.7202518613225 }, { 55.71921345792441, 124.49167711799818 }, { 401.5613613735741, 42.764836970428576 }, { 201.6137501876498, 336.8465615314824 }, { 150.36377621889721, -15.309070160739083 }, { 426.22420352492463, 209.89797856242126 }, { 70.44004546019818, 230.19243494767935 }, { 319.24880152724836, -25.278654400663328 }, { 308.34556099995166, 331.41370182379046 }, { 75.28125926571022, 60.84690321520998 }, { 430.1469353089999, 102.95102868244358 }, { 139.8380319861747, 311.7576157000172 }, { 212.90655253014052, -38.5529418097733 }, { 395.78571392347726, 269.3248806176251 }, { 52.731275014011715, 165.76697640142365 }, { 375.8638600179259, 10.290396156554436 }, { 242.52710967074447, 343.43599940651984 }, { 115.72183243628629, 7.486372778504716 }, { 436.3746961152916, 169.66072196352073 }, { 90.150908146776, 266.74272987931477 }, { 279.9975043246476, -38.92236000817951 }, { 346.5297139374465, 314.946814295246 }, { 58.23913521458633, 98.80907955069313 }, { 417.05900416721056, 63.42078956394434 }, { 176.21696472763568, 332.0770630123467 }, { 172.3291074308106, -28.953431723436864 }, { 419.2359446950542, 234.81199465250694 }, { 58.75932597369456, 207.08390049603543 }, { 343.48630426924814, -16.119687348869746 }, { 284.2692747350062, 341.03530938040956 }, { 86.54597399624242, 37.474937077850214 }, { 437.6296292734119, 127.81167284024984 }, { 117.51443027870475, 298.4685633473905 }, { 238.36161660112, -43.83324656084358 }, { 380.5695960484824, 290.4249159752477 }, { 49.69997594992921, 139.91252181501596 }, { 395.5741280560478, 27.32115717780445 }, { 216.4718896578646, 344.188939380787 }, { 134.4404948072251, -10.678236746154312 }, { 434.8374579594248, 195.71601395347867 }, { 73.67630625146433, 246.47628357346602 }, { 305.85165366207053, -35.10079527563079 }, { 324.86792701446427, 329.5997475677632 }, { 64.3215045934258, 73.3562535826656 }, { 429.772346642429, 86.31429564018907 }, { 151.36305795414373, 323.7749442962962 }, { 196.28077019878708, -39.62394702183582 }, { 408.7723922759683, 258.8735061392626 }, { 50.21935276984516, 182.25626731931732 }, { 366.5675708859943, -3.569869859418077 }, { 258.7550940564631, 347.37326464709867 }, { 101.0905736972667, 15.554671433195466 }, { 441.7109235498592, 153.8171881356375 }, { 96.92741251652981, 282.0363121542347 }, { 264.65895868559255, -45.62053449977631 }, { 362.37168283530855, 309.5163014969098 }, { 50.22640978461692, 113.5256237474874 }, { 413.0186399110554, 47.148790223426616 }, { 190.19891227140192, 341.34689484823775 }, { 155.74896901565185, -26.336973747662512 }, { 429.6957994627895, 221.67195841613284 }, { 59.92880028696567, 223.84766452723068 }, { 331.2893886916294, -27.69332610903257 }, { 301.09046410635335, 341.325014759139 }, { 73.94339163709901, 48.63440858065847 }, { 439.3797098837084, 111.05999480030749 }, { 127.54975071436087, 312.00726799108975 }, { 221.80616455144846, -47.033861290406975 }, { 394.94430326765223, 281.59216249630464 }, { 45.06965756201072, 156.14621438214422 }, { 388.0134527081253, 12.217125978033721 }, { 232.26042305008613, 350.2172722440792 }, { 118.71958607227533, -4.449698250488012 }, { 442.2214705685998, 180.49296675183535 }, { 78.52257916873802, 262.69669224772656 }, { 291.31042902047807, -43.78780167680131 }, { 341.46640302334674, 326.1755354137789 }, { 54.39416813158485, 87.10419999214898 }, { 427.79933059392886, 69.46233086599423 }, { 164.21200647211543, 334.87041925458186 }, { 179.30210605217417, -39.12035619287093 }, { 420.95480956118934, 247.02272515233435 }, { 49.246359890735164, 199.23358483818438 }, { 355.8068106264478, -16.74962142858365 }, { 275.6027284471043, 349.8188481434536 }, { 87.01084984073907, 25.14158712042088 }, { 445.6139563090271, 137.2268931700169 }, { 105.26534299035868, 296.9116106339751 }, { 248.4520112844916, -50.954814677967136 }, { 377.93189596617754, 302.49322398387005 }, { 43.49797273936662, 129.22582145945614 }, { 407.36648931859105, 31.01976201236191 }, { 205.2725046580664, 349.42176419834675 }, { 139.17179894906823, -22.10158227307963 }, { 439.0590784377637, 207.34036458788682 }, { 62.71221368427061, 240.74863229472277 }, { 317.80983990315997, -38.27710865516127 }, { 318.18814723969035, 340.0178931936272 }, { 62.21640429887967, 61.15805859713001 }, { 439.5620786543882, 93.89446103973009 }, { 139.0206421101253, 324.79133163863713 }, { 204.70245569234612, -48.70767732571369 }, { 408.6911163436604, 271.26314560286835 }, { 41.913720349293726, 173.05859214142998 }, { 378.90698901482097, -2.390606218908289 }, { 248.85311405644268, 354.83476550318505 }, { 103.35951496440003, 3.362601679432373 }, { 448.26884496098785, 164.34619273687028 }, { 84.97873665116703, 278.694578660421 }, { 275.73271570719106, -51.2224862894177 }, { 357.9824912402948, 321.1273750877771 }, { 45.62534332225161, 101.99373545453652 }, { 424.2004816203233, 52.55185010160781 }, { 178.29918526456578, 344.9099930759061 }, { 162.12430966195672, -37.00125462092856 }, { 432.19197638936635, 233.8462669999131 }, { 49.86641213300112, 216.54930735469486 }, { 343.64278770323256, -29.101991900322417 }, { 292.9256499144631, 350.70584086422593 }, { 73.6342957118606, 36.19889571688721 }, { 448.0045964702832, 120.02785023460575 }, { 115.12982526082982, 311.213367418719 }, { 231.50728229768666, -54.83431091900292 }, { 393.0526449539156, 293.89884152177024 }, { 38.15571990600759, 145.78743300741826 }, { 400.110035640932, 15.192672608850046 }, { 221.32480703277167, 356.1895376738671 }, { 122.75662342396606, -16.240906046432855 }, { 447.20425666441196, 191.91998902658946 }, { 67.1297054839867, 257.6289204343731 }, { 303.139497199301, -47.74103543858942 }, { 335.406776438281, 337.0801678279526 }, { 51.5025050807032, 74.9656794858732 }, { 438.1295746826737, 76.46710825226805 }, { 151.85913584154247, 336.67680535880567 }, { 187.19786674915747, -48.794504269640385 }, { 421.6607170860176, 259.49307985807127 }, { 40.30493125204501, 190.50805226185986 }, { 368.29677864743485, -16.348479200370008 }, { 266.11520422042946, 357.9566236195799 }, { 88.51692431473981, 12.73011371002454 }, { 452.88339581703224, 147.40258703599312 }, { 93.02993934202584, 294.311400191003 }, { 259.2365857840797, -57.297998895718536 }, { 374.256946590987, 314.45628179239503 }, { 38.13173978900076, 117.91650424452276 }, { 418.9629809304599, 35.74159806237323 }, { 193.52677500099134, 353.7679767684909 }, { 144.9044261666384, -33.240084108272185 }, { 442.3501723562664, 219.43067396881975 }, { 52.11960878452123, 234.04927588386641 }, { 330.15011171898345, -40.486238191829585 }, { 310.5737506101409, 349.98090299748503 }, { 61.10761515923045, 48.66448597706696 }, { 448.81666426287893, 102.36498144293043 }, { 126.4717178629301, 324.7900249844131 }, { 213.96341245395138, -57.180501053093536 }, { 407.5786625236818, 283.7689521935064 }, { 34.289925035268226, 163.07914747724385 }, { 391.271460634669, -0.1746730981001292 }, { 238.23283158875705, 361.5488695797326 }, { 106.66259343157307, -8.76334028866583 }, { 454.0196798482837, 175.52456577914566 }, { 73.18673317185846, 274.32923375564525 }, { 287.38197988723095, -55.9639848766796 }, { 352.5881155416794, 332.4925668564425 }, { 41.931369906910334, 89.96455147807578 }, { 435.04925451899, 58.93391808871459 }, { 165.9841143598574, 347.5264365950691 }, { 169.44494615311288, -47.24797293928171 }, { 433.70931141666154, 246.3508653438533 }, { 40.30273787452194, 208.34663257666801 }, { 356.2389810693193, -29.507242082317276 }, { 283.9044363157743, 359.5107666831069 }, { 74.3453956647252, 23.609846086445458 }, { 455.9809524962668, 129.79762607459006 }, { 102.64669097942308, 309.3904037361002 }, { 241.9497099568645, -61.91871109691351 }, { 390.1310277159332, 306.1780303133892 }, { 32.01972407427965, 134.75360760932517 }, { 412.08875938097754, 19.191105768845517 }, { 209.7855491259335, 361.32806649552725 }, { 127.80157140217293, -27.824913608519296 }, { 451.3040323928162, 203.87467296942208 }, { 56.03165612806308, 251.5759917825959 }, { 315.41626154849433, -50.76887868627142 }, { 328.39233918572336, 347.6047437038692 }, { 49.57140015000692, 62.46279072865522 }, { 447.9975775720782, 84.38900163491911 }, { 139.22793924114302, 337.4949411488062 }, { 195.96600058554571, -57.92790538712586 }, { 421.35836236729915, 272.15367212986314 }, { 31.97847422919611, 180.96158322393345 }, { 380.8875329497023, -14.926888384936944 }, { 255.86446840980918, 365.41005236778835 }, { 91.0477233974471, 0.3079752261132853 }, { 459.4044933640383, 158.27793309874357 }, { 80.87396443190417, 290.6901230074311 }, { 270.65191678996956, -62.83471397924143 }, { 369.5724845184108, 326.25054074395223 }, { 33.62379277692125, 106.05012368762058 }, { 430.30271716497657, 41.453595228261534 }, { 181.30189738896283, 357.2107900428474 }, { 151.6002241107733, -44.03596584356711 }, { 444.6997979360764, 231.9186008902583 }, { 41.95273563437428, 226.42116041026372 }, { 342.8034900536683, -41.723160748302234 }, { 302.07215326275, 359.43846174284977 }, { 60.994212081275776, 35.94464174568162 }, { 457.49008419814635, 111.67432701090223 }, { 113.78500686611432, 323.7778832714454 }, { 224.008383370188, -65.00102993038371 }, { 405.44776537041724, 296.3230966420856 }, { 27.384423622763563, 152.37650409794645 }, { 403.5945498024288, 3.0594209456839963 }, { 226.95569334889407, 367.48431529624054 }, { 110.97567076071604, -20.75860202427151 }, { 458.93758690767277, 187.28844324282625 }, { 61.61359524742332, 268.96985981969624 }, { 299.54094594529823, -59.82479398083828 }, { 346.2233563035068, 343.55250079520874 }, { 39.15897770654374, 77.50532836441063 }, { 445.5088363682347, 66.25523971758649 }, { 153.32190707692124, 349.1875857484837 }, { 177.66647084506317, -57.02467234672136 }, { 434.24507038677905, 259.11710183609995 }, { 31.286183400898864, 199.28836609061193 }, { 369.00938279865665, -28.912261179528826 }, { 274.07950084077845, 367.6957910150221 }, { 76.06761898310478, 10.935271292261461 }, { 463.2695027035819, 140.31308873952997 }, { 90.16728103105484, 306.5532631253891 }, { 253.07407949492622, -68.25279843376546 }, { 386.20003403912176, 318.36436975911175 }, { 26.69067309221225, 123.10679620448816 }, { 423.88632235580644, 24.18630618815773 }, { 197.70693329314713, 365.60932529754757 }, { 133.82297441633816, -39.14094470006836 }, { 454.5029024275413, 216.29098881711778 }, { 45.28623336127603, 244.57445830587193 }, { 328.0734391924941, -52.85920064644429 }, { 320.4644621733138, 357.6948609246225 }, { 48.607136010358175, 49.66356814490935 }, { 457.35273783184255, 93.18213429007275 }, { 126.38669593310671, 337.324432092567 }, { 205.55647130321321, -66.47413076183719 }, { 420.0532352626177, 284.9364859605969 }, { 24.30890432775834, 170.64799662175855 }, { 393.5118264808452, -12.496175747749305 }, { 244.90771786142915, 372.1420389498395 }, { 94.58617425176601, -12.058849783881612 }, { 465.14524426639434, 169.79278602563582 }, { 68.86163861092672, 286.07046450120527 }, { 282.6353615108338, -67.5382785930941 }, { 363.90663261880013, 337.8140148818625 }, { 29.995296898094523, 93.69135081167565 }, { 441.32640563005566, 48.12240862722801 }, { 168.6640871504473, 359.73470654659224 }, { 159.2208800121124, -54.43306738841622 }, { 446.0982346853042, 244.73688073451422 }, { 32.26420697226601, 217.9072756309348 }, { 355.7021634167221, -41.98399897245011 }, { 292.7306964562773, 368.34189233184895 }, { 61.87420669456802, 23.066378193906445 }, { 465.53795565357035, 121.77116261327785 }, { 101.02789625300124, 321.7627394580551 }, { 234.78242653200917, -72.12950180102496 }, { 402.31203775991673, 308.85915904833547 }, { 21.23206596417816, 141.0087970355822 }, { 415.81129830547616, 7.292390817947478 }, { 215.08259037051974, 372.6114199002923 }, { 116.27393954056194, -32.560170155805 }, { 462.9982665459913, 199.57463540871169 }, { 50.319923151205614, 262.6465956175511 }, { 312.1445995669394, -62.78617290172531 }, { 338.9234530667761, 354.2494396070706 }, { 37.321214407965414, 64.68241770331052 }, { 455.52478100162654, 74.47574312182051 }, { 140.3797553733665, 349.88617739253266 }, { 186.74428384591332, -66.28060105602276 }, { 433.79782297948, 272.07744693370165 }, { 22.863430772080704, 189.42329795516048 }, { 381.8866306652585, -27.3214563893672 }, { 263.5034548374269, 375.2186461889638 }, { 78.7907598544443, -1.7581370088898325 }, { 469.83270522948317, 151.51820271734647 }, { 77.75711865749267, 302.71786782958486 }, { 264.82135362768946, -73.80403274508342 }, { 381.2811777676821, 330.3939937566745 }, { 22.195867186100003, 110.90859335901827 }, { 435.4409915602879, 30.151334939225876 }, { 185.1526481268787, 369.01145274225564 }, { 140.78867539491722, -50.12985905478817 }, { 456.78460243567247, 229.1036324999247 }, { 34.949544811320635, 236.66144942751657 }, { 341.04460084173286, -54.001450133407275 }, { 311.664964389666, 367.29786138152724 }, { 48.61447737673828, 36.6350679496237 }, { 466.14623762908144, 102.80031531001218 }, { 113.40258645692032, 336.1663299255083 }, { 215.91906618674463, -74.38856456189717 }, { 417.7521903930758, 297.7747196323228 }, { 17.33630537488105, 159.62114703838637 }, { 406.1037362713123, -9.0689418116568 }, { 233.30204211609953, 378.1173273737933 }, { 99.11403040626502, -24.30581006749739 }, { 470.075479829345, 181.88725213807112 }, { 57.05565908967421, 280.47617494132567 }, { 295.12467362267324, -71.38383251079108 }, { 357.2884616968358, 349.086329113456 }, { 27.265622665671145, 80.90431581265513 }, { 451.97644368117335, 55.71386989727988 }, { 155.67887581856374, 361.32598546328603 }, { 167.727388954863, -64.37698776786925 }, { 446.5374691796777, 257.81907330594515 }, { 23.104817194772977, 208.55116596239645 }, { 368.779282068078, -41.26652153055883 }, { 282.59713325031277, 376.6443845218082 }, { 63.74403692771662, 10.096452788069342 }, { 472.91779996778416, 132.60388871207687 }, { 88.26655046036379, 318.7539168898883 }, { 246.23048666341072, -78.52808427625314 }, { 398.186488999932, 321.31206394529556 }, { 15.86575856223692, 129.03412093277643 }, { 427.8582070554894, 12.503659154559841 }, { 202.67421779628393, 376.9024452327983 }, { 122.53141394300263, -44.106597200305 }, { 466.1793524508766, 212.32030469242466 }, { 39.36465327429025, 255.39063538146038 }, { 325.1284345297047, -64.83129021639965 }, { 330.72456946299894, 364.52739830028133 }, { 36.429255164254954, 51.56150836960286 }, { 465.04447266135617, 83.55457140649746 }, { 127.2240362621977, 349.6167714733333 }, { 196.6331461720598, -74.9669047809819 }, { 432.367899949098, 285.165334811864 }, { 15.079209092831803, 178.8007053834002 }, { 394.8044704199191, -24.74092161774209 }, { 252.2292398035753, 382.03906074784845 }, { 82.50301216324294, -14.40493383385882 }, { 475.635045006961, 163.35676343165278 }, { 65.48034831865752, 297.9016423517795 }, { 277.1324901402302, -78.54191814632259 }, { 375.3973658175282, 342.2045395064548 }, { 18.56055817100801, 98.22043246140127 }, { 446.6926532625263, 37.05797977719945 }, { 172.18605406486176, 371.5146263957663 }, { 148.6653736049546, -60.73424218113598 }, { 458.1348837288906, 242.24779481731764 }, { 25.07587616601768, 227.87509942128713 }, { 354.2639773790479, -54.18705455768202 }, { 302.0365225318928, 376.36299392618946 }, { 49.59626114526429, 23.443523451340994 }, { 474.3312392246816, 113.19665065427968 }, { 100.34180943920387, 334.0235498547185 }, { 227.00303043272808, -81.62863162632698 }, { 414.4638681930863, 310.6027074682344 }, { 11.09867309638065, 147.93526465764438 }, { 418.5986235089696, -4.659482301322356 }, { 221.104735467321, 383.30278352729977 }, { 104.61145329593259, -36.36977952550629 }, { 474.16717294838026, 194.50140708412474 }, { 45.51716512448965, 273.9324816384457 }, { 308.0577522688614, -74.34866241838841 }, { 349.7483932503914, 360.00878987479797 }, { 25.45200296847358, 67.7528264193614 }, { 462.1970235698982, 64.19259689024591 }, { 142.41129582647315, 361.97299958180145 }, { 177.0796788778742, -73.81522314183812 }, { 446.0115628344891, 271.0994125334855 }, { 14.52337330788771, 198.39728749862303 }, { 381.9688425614577, -39.570518506180264 }, { 271.7199650701249, 384.30119295324954 }, { 66.59818717367466, -2.8993913690180193 }, { 479.58926739340495, 144.12032750579888 }, { 75.56595682280155, 314.7626104117595 }, { 258.2971060004256, -84.1611238393105 }, { 393.087904116956, 333.618069166047 }, { 11.316193679721039, 116.51079264421443 }, { 439.6732587427668, 18.67098188109304 }, { 189.79130942582043, 380.3318877339783 }, { 129.72056304753784, -55.33805707468355 }, { 468.46071794493963, 225.46276042599894 }, { 28.80497093246305, 247.23458492670534 }, { 338.42827782759974, -65.9455436408796 }, { 321.6641361193675, 374.3322616578389 }, { 36.49207211503136, 38.2077703082209 }, { 474.0172721245113, 93.44975544220972 }, { 113.92042402501207, 348.37608921006375 }, { 207.28687055116146, -83.03679970277122 }, { 429.95773609608045, 298.3150829391484 }, { 7.976096701556685, 167.4706418149896 }, { 407.6977071091089, -21.178782345829376 }, { 240.31039442511002, 388.1189799502687 }, { 87.19062557688426, -26.940903635344824 }, { 480.64327466430905, 175.77215392862138 }, { 53.399722035613536, 292.12385363604335 }, { 289.9482237624081, -82.43826194014443 }, { 368.5732319313594, 353.73519127589606 }, { 15.807675081524422, 85.10377742924516 }, { 457.58288586056534, 44.87643039513023 }, { 158.87034768989707, 373.1013499999001 }, { 157.41831137523582, -70.89850608108458 }, { 458.54181080743194, 255.65902529128175 }, { 15.717564023628285, 218.2548469066486 }, { 367.6663523678684, -53.409725082289526 }, { 291.622954565578, 384.84156633454654 }, { 51.55308629137065, 10.154423630814108 }, { 481.86305996640976, 124.32327750454102 }, { 87.26963124183598, 330.9011829541804 }, { 238.7568212122297, -88.15399355094823 }, { 410.1990069826863, 323.3558988961345 }, { 5.631700006682536, 135.64517989891016 }, { 430.9331408652871, 0.7159026673969038 }, { 208.37349890487317, 387.66762713014793 }, { 111.0567084397916, -48.18909213013197 }, { 477.39468462947366, 207.5751177606224 }, { 34.30567639000057, 266.46638501084146 }, { 321.3724888030214, -76.41246065474158 }, { 341.31848550288373, 370.5244723838558 }, { 24.56926451076663, 54.30049559749055 }, { 471.9342440510559, 73.52171982167762 }, { 128.92598215064066, 361.66651124462834 }, { 187.2363509118382, -82.69730132041511 }, { 444.51693279764925, 284.51273117155813 }, { 6.566539108420841, 187.49125163866145 }, { 395.20563885046613, -36.898084803312685 }, { 260.1486676204368, 391.26981252399423 }, { 70.42890478265372, -15.856402891731136 }, { 485.51433015296055, 156.26751622843432 }, { 62.98992234226819, 309.802167619937 }, { 270.92623811333124, -88.99535460131429 }, { 387.03512056090415, 345.714794374123 }, { 7.611626955474151, 103.4975157765129 }, { 451.1959708922467, 25.77017826512808 }, { 176.49477970408844, 382.8767123123931 }, { 137.81204976000762, -66.1964217156303 }, { 469.8247191443888, 238.93934012986142 }, { 18.696211320129805, 238.21271187237028 }, { 351.9801947053991, -66.1168102495522 }, { 311.78108815454084, 383.6119044086939 }, { 37.51617934948928, 24.6859254425909 }, { 482.39465654876653, 104.11799081627044 }, { 100.53393697372215, 346.16327071053996 }, { 218.65811438835837, -90.44573040458951 }, { 426.5721282448785, 311.46186934926504 }, { 1.5943468515224595, 155.4841265309671 }, { 420.502206069305, -16.64545179446074 }, { 227.80122554041247, 393.4227546655169 }, { 92.83765331287611, -39.30308451617273 }, { 484.8266161022823, 188.70719361465942 }, { 41.57655264287115, 285.40585732454497 }, { 303.2089354755121, -85.46738691260711 }, { 360.83537500578484, 364.9267488192106 }, { 13.957603171136753, 71.62023215558641 }, { 468.05504894811986, 53.57504987700791 }, { 145.26864944195725, 373.75668112876576 }, { 167.01105706202495, -80.56899778283127 }, { 457.995993607876, 269.2731664840685 }, { 6.924869212734933, 207.84163876765115 }, { 381.1870185050073, -51.66569139813453 }, { 280.4694090664049, 392.68708868780266 }, { 54.483078612169464, -3.1674653640718304 }, { 488.69933081135025, 136.13119081163535 }, { 74.25038567120362, 326.8067298942708 }, { 251.12795125929796, -93.92672069528257 }, { 404.9706732095567, 335.97088007988566 }, { 0.9685916566838273, 122.80646179979897 }, { 443.0452748510427, 7.0386837257757975 }, { 195.16655965205783, 391.18362542619263 }, { 118.42594777134956, -59.70360422526443 }, { 479.7349657653673, 221.0479422634645 }, { 23.479011668036122, 258.1068673448441 }, { 335.006686692233, -77.55753318311596 }, { 332.0326316450678, 380.5783198854139 }, { 24.629615551565678, 40.610801405822485 }, { 481.13622635256826, 83.66269503814917 }, { 115.28721181561406, 360.39988699045205 }, { 198.15450651666623, -90.9749131302805 }, { 442.052567082009, 297.9944415827509 }, { -0.7213101443798564, 175.87998335684523 }, { 408.4252631235597, -33.25383390698909 }, { 247.93383348741517, 397.51013615255806 }, { 75.22598948896692, -28.710842597438983 }, { 490.65745103239465, 168.99158113847056 }, { 50.60103411294608, 303.88829449243053 }, { 284.0611392512238, -93.00007605685434 }, { 380.0492272111053, 357.54127901382014 }, { 4.777692202724381, 90.0534713854529 }, { 462.36747133085873, 33.77494012906415 }, { 162.8457960220807, 384.516543243977 }, { 146.77454973979263, -76.62535307151313 }, { 470.2563897892837, 252.68735588174877 }, { 9.09175211007053, 228.36111561404226 }, { 365.72045380551117, -65.3356434683221 }, { 301.11602278529807, 392.3163128079199 }, { 39.50543550588432, 11.060263804449818 }, { 490.13035390409505, 115.51449372956148 }, { 87.12893507148428, 342.98007105101556 }, { 230.69825016806908, -97.15151573947139 }, { 422.21842831659404, 324.54175315924977 }, { -4.028268103970333, 142.89325888782858 }, { 433.1549310648095, -11.153819358449226 }, { 214.75690629838155, 397.91730558656093 }, { 99.42576990578831, -51.42982332445112 }, { 488.1569315106816, 202.10405673545915 }, { 30.070642533093093, 277.7712723068187 }, { 316.8545880361446, -87.60630752744788 }, { 352.21252456600064, 375.72171386965 }, { 13.028004183184862, 57.831587686687556 }, { 478.0543840045774, 63.12021958634594 }, { 131.44403339816958, 373.4684128068859 }, { 177.40536147809075, -89.69411308390252 }, { 456.4907693759926, 283.02634156311626 }, { -1.2541487653789716, 196.67806112607408 }, { 394.76178232800805, -48.95388224232741 }, { 268.6224826650323, 399.85540942406953 }, { 58.38171313342562, -16.458121720087092 }, { 494.80014148172575, 148.57014017646225 }, { 61.34743617023966, 321.7502740900371 }, { 264.0629007373912, -98.91144499633376 }, { 398.79442869664604, 348.3854274113336 }, { -2.860092348646191, 109.47549064568707 }, { 454.8744142434201, 14.287942286094165 }, { 181.5427279162353, 393.82525681533303 }, { 126.69305824903886, -70.85477702171397 }, { 481.1677238818827, 234.85908924638602 }, { 13.093193260247858, 248.88503440837326 }, { 348.8980370709546, -77.76896791381384 }, { 321.92669057758525, 390.1172512555496 }, { 25.642477533213736, 26.74709530431622 }, { 489.7532326337132, 94.57518612827374 }, { 101.55889626175787, 358.16926447034757 }, { 209.78964130945377, -98.60204048646202 }, { 438.62018855841194, 311.4805589761282 }, { -7.298224108644149, 163.61181391940335 }, { 421.5641441822555, -28.645057533526938 }, { 235.12725079085513, 402.9845981076203 }, { 80.97663946343147, -41.400036432548546 }, { 494.9857322613191, 182.23767340724004 }, { 38.46059255844011, 297.03920234479875 }, { 297.6443191118856, -96.14730608835407 }, { 372.1537031320456, 369.0380620875102 }, { 2.8372457223596825, 76.23835211978343 }, { 473.1305900545725, 42.656702412521426 }, { 148.9057977055445, 385.2338210970013 }, { 156.57463229982, -86.57040604131083 }, { 469.74359797278294, 266.6440904170454 }, { 0.0429005434590124, 217.71783517285863 }, { 379.58553815898597, -63.59542819665424 }, { 289.71129512389314, 400.39770634460024 }, { 42.4608916337703, -2.605382020588678 }, { 497.180472244275, 127.59291793358435 }, { 73.76908026637011, 338.83100809850606 }, { 243.3572956857615, -103.1144843815213 }, { 416.9066855615879, 337.4917277774404 }, { -8.856597220870697, 129.7512743563774 }, { 445.5940103549275, -4.719386042424958 }, { 201.23351816104568, 401.57226734990786 }, { 106.9341436983895, -63.26085379270748 }, { 490.60886973760705, 215.90424458765892 }, { 18.94019432984402, 269.2460992571728 }, { 330.824712451462, -88.83487718185606 }, { 342.73564938099986, 386.06438968336676 }, { 13.033669561214282, 43.79982156144773 }, { 487.5281231058808, 73.47624132305737 }, { 117.45951263036754, 372.2272185019766 }, { 188.56107156453413, -98.22441331151575 }, { 454.02234456517215, 296.8549825999328 }, { -8.773755371672365, 184.8084135239827 }, { 408.327005727431, -45.276062948973475 }, { 256.1302815697253, 406.30484462660684 }, { 63.24168210508185, -29.654322129006204 }, { 500.12817415000467, 161.5885813821314 }, { 48.62310965146875, 315.7446070251417 }, { 277.5070820996603, -103.0754967703956 }, { 391.68844822499796, 360.5385849133362 }, { -5.826576976366482, 95.7094796304169 }, { 466.36143787332185, 22.440261678445637 }, { 167.56140504202085, 395.5698500107198 }, { 135.82956260147893, -81.58577383426913 }, { 481.6755615436213, 248.94742318252844 }, { 3.2023413094594844, 238.83420430488738 }, { 362.9841372055615, -77.03477109745714 }, { 311.0385646741505, 399.0902740497785 }, { 27.614351928893768, 12.772570814285388 }, { 497.73778527683976, 106.21699783297026 }, { 87.80453756956041, 354.9736811740571 }, { 222.0955911196489, -105.5350807583417 }, { 434.2243781232603, 324.9077570727077 }, { -13.124805598286969, 150.73652232713175 }, { 434.559614349574, -23.08184216918559 }, { 221.78193188355917, 407.6583048952391 }, { 87.6653424027528, -53.862452883104794 }, { 498.46904781473154, 195.9499532869287 }, { 26.62852434632876, 289.27570819270926 }, { 311.6175378843461, -98.41191344824065 }, { 363.37450855426613, 380.14727801028863 }, { 1.8102348632396854, 62.11235241275364 }, { 483.4299628770149, 52.38456231382898 }, { 134.73647366295583, 385.0139319281211 }, { 167.17669031267084, -95.97913888166192 }, { 468.27717192453053, 280.7468316913875 }, { -8.401222649482037, 206.32290798573612 }, { 393.5121915190857, -60.892502051412464 }, { 277.6110652325311, 407.8106583667908 }, { 46.380675564931096, -16.24769445621314 }, { 503.5036237205615, 140.30531973406636 }, { 60.5172683359626, 333.7234715105252 }, { 256.583891320388, -108.2976010516334 }, { 410.6497479843738, 350.24979892419674 }, { -12.858238216290545, 116.1125548821451 }, { 457.75882377078926, 2.6396425578547564 }, { 187.2880489108174, 404.3601156292036 }, { 115.33935308890761, -74.73739178715508 }, { 492.1599920520241, 230.04859973959188 }, { 8.241708415530212, 259.8587944638252 }, { 345.0584341936385, -89.13591148731751 }, { 332.4380207693862, 395.9009897275624 }, { 13.986400768539141, 29.587059943641222 }, { 496.4256027743715, 84.60528504087098 }, { 103.37799012669637, 370.02676712790145 }, { 200.43608952793622, -106.11274324173186 }, { 450.5899041355992, 310.6958903189511 }, { -15.590661897384905, 172.27873807641475 }, { 421.8196757340406, -40.63693807770164 }, { 243.04243882623737, 411.99630070239357 }, { 69.05279994273559, -42.69372089533712 }, { 504.6488267819124, 175.133671100952 }, { 36.13860886518779, 308.8053149159447 }, { 291.40484672847685, -106.38902757685136 }, { 383.6735970605994, 372.37075952818054 }, { -7.905993019014886, 81.56645642585401 }, { 477.44881664003094, 31.469659416321477 }, { 153.2825535368091, 396.39770257313273 }, { 145.80456194793848, -91.84156783805997 }, { 481.24409109258517, 263.25150553373703 }, { -6.14143866873485, 227.98995403265752 }, { 377.2025424107221, -75.3459772324887 }, { 299.40823515567456, 407.44860061811335 }, { 30.548716151432217, -1.2497986034052246 }, { 505.0447855576279, 118.54405211833406 }, { 74.08715676255412, 350.81517177823326 }, { 235.02451966608876, -111.73296688739057 }, { 428.8726644434913, 338.2134486653013 }, { -18.164331034100428, 137.30533598240652 }, { 447.34999810457293, -16.577150425064247 }, { 207.95210241579397, 411.49915493266235 }, { 95.27380327153173, -66.03779882782018 }, { 501.0801612121578, 210.0716124912179 }, { 15.163280233004116, 280.62129729193344 }, { 325.92183981511573, -99.77173260888378 }, { 353.7401371659719, 390.81276409917103 }, { 1.7135871638025435, 47.73612408896305 }, { 493.21214324171024, 62.92523768233225 }, { 120.39970761956921, 383.84531314807657 }, { 178.54290962417315, -104.80122829741057 }, { 465.8509999323079, 294.93293747438827 }, { -16.193800549238915, 194.21838923372152 }, { 407.43749215214257, -57.22624845100961 }, { 264.86130605903776, 414.51221450658636 }, { 51.259906494720326, -29.80395321315035 }, { 509.0610430429695, 153.6021614797946 }, { 47.43553890594973, 327.66779973269 }, { 270.3253148769792, -112.66658393032506 }, { 403.46333024331443, 362.75508146807226 }, { -16.003652762169224, 102.03260286122 }, { 469.59010549947016, 10.902298046285324 }, { 172.97835571637, 406.25627926230914 }, { 124.61533851575103, -85.80224308328081 }, { 492.79088008923304, 244.477353507141 }, { -1.9701288381433528, 249.6403072398967 }, { 359.49453075630856, -88.49529105963512 }, { 321.3552390920571, 405.17975241439 }, { 15.894912577957484, 15.255510534504964 }, { 504.69838062454977, 96.467373488318 }, { 89.26218280611954, 366.8638128083946 }, { 212.98636873022969, -113.31434859100582 }, { 446.1956935694242, 324.48631827083716 }, { -21.66414273114742, 159.13681231816622 }, { 435.1774960988185, -35.04422494991201 }, { 229.41009598728726, 416.8933903598138 }, { 75.80193882294026, -55.514948078209784 }, { 508.33032677426814, 189.1512962168682 }, { 23.953908596616742, 300.9508334152107 }, { 305.69952496700597, -108.82512048944696 }, { 374.77347547552756, 383.8238296324845 }, { -9.076483005224759, 67.10521281851379 }, { 488.080725749738, 41.34755324780117 }, { 138.7666368438792, 396.2921815080593 }, { 156.58471271674387, -101.56905701153906 }, { 479.8620290282479, 277.7096642861461 }, { -14.888172392505226, 216.3901315014369 }, { 391.4908442574118, -72.69673638192583 }, { 287.0777629429145, 415.1457653216827 }, { 34.44594408179219, -15.257345716891507 }, { 511.63163041499445, 131.51039848692108 }, { 60.469200459311054, 345.69883906179683 }, { 248.52693993322762, -117.15728265946981 }, { 422.5755856819145, 351.3358853026982 }, { -22.382863182616973, 123.37089844477471 }, { 459.8747172047663, -9.146873020906895 }, { 193.69315851527068, 414.47794774196745 }, { 103.78090252454575, -77.86712915407918 }, { 502.7948301870929, 224.54492720532397 }, { 4.121721937096623, 271.1021543362209 }, { 340.49761593320653, -100.20766297011508 }, { 343.281636451144, 400.9801761975025 }, { 2.5611174948703592, 33.17070448450578 }, { 502.42571931074275, 74.24305772389299 }, { 105.95749772293416, 381.71953919710063 }, { 190.63327088889451, -112.98858694912079 }, { 462.46210806824627, 309.1399235442083 }, { -23.290436925257097, 181.44833888815174 }, { 421.29894804968797, -52.599165695603546 }, { 251.50977947406236, 420.4620078873793 }, { 57.090635168189266, -43.21213667525063 }, { 513.8166999902268, 167.4323463283592 }, { 34.58496878733942, 320.6773300077185 }, { 284.5275268546457, -116.19001346835367 }, { 395.36605340248883, 374.9479104222498 }, { -18.266270627449842, 87.56798576259968 }, { 481.03005843346693, 20.044820093855577 }, { 158.36310012634794, 407.2392387971955 }, { 134.73338421334904, -96.39992005873145 }, { 492.4852279218766, 259.1302000235552 }, { -11.642563076584679, 238.62408710537076 }, { 374.071514143672, -86.90204629320971 }, { 309.5252297948366, 413.8510593796733 }, { 18.764756352243495, 0.8673723820447208 }, { 512.300352870553, 109.02039728532044 }, { 75.17452441102907, 362.73826285401753 }, { 226.16593976744466, -119.7869917155464 }, { 440.8450774935207, 338.1640760113256 }, { -26.95614822895186, 145.43211233580834 }, { 448.3389956449755, -28.508702387013955 }, { 215.28585576220024, 420.9625416523495 }, { 83.47299027397631, -68.05772207305 }, { 511.14383520846025, 203.58613126010556 }, { 12.127639835170413, 292.20247533742895 }, { 320.33349315231067, -110.35988863682209 }, { 365.0144355270441, 394.8412631077319 }, { -9.319293153700073, 52.38522864734229 }, { 498.20316399667877, 52.04275549021773 }, { 124.07453489898623, 395.23980781609214 }, { 168.13423216506214, -110.71718358790483 }, { 477.52127240092875, 292.2600860538643 }, { -22.99013539298784, 204.07483885294945 }, { 405.7867695491789, -69.08438172627274 }, { 274.0912609391226, 422.1377412039511 }, { 39.30324796421672, -29.187814524446225 }, { 517.4583261941804, 145.06825251561992 }, { 47.01243074789684, 339.63290203616793 }, { 262.55176321865656, -121.77237252396961 }, { 415.3467272117664, 364.2142715941201 }, { -25.749356088327943, 108.98721027554853 }, { 472.074408187028, -0.8098557322070121 }, { 179.0615980130658, 416.5684830734573 }, { 113.16268018658471, -89.29296644425895 }, { 503.591899128604, 239.3113359062562 }, { -6.44099870128133, 260.74716818553406 }, { 355.28469026566904, -99.70375380245338 }, { 332.0326011567757, 410.5971096280033 }, { 4.363451368035129, 18.477422559932904 }, { 511.0214339856521, 86.29998076719244 }, { 91.47185577486414, 378.6313893014655 }, { 203.40557844620494, -120.49548150397769 }, { 458.1107184024577, 323.30557048604476 }, { -29.649271294424466, 168.05878194391016 }, { 435.0346088540465, -47.016915110773795 }, { 237.6059859262053, 425.62237019161444 }, { 63.86180624809333, -56.41103800366207 }, { 517.7374055210872, 181.74327876573722 }, { 22.02555267937356, 312.76842514311505 }, { 299.13524064997, -118.83943261667977 }, { 386.3794605745828, 386.7699624200627 }, { -19.622583150448747, 72.77625681527219 }, { 492.0224767899497, 30.04065208663799 }, { 143.50166029976762, 407.29061238659403 }, { 145.6621252212411, -106.4767644101041 }, { 491.22991961983456, 273.9463917397874 }, { -20.724893727804897, 226.8460654432131 }, { 388.72773330861946, -84.34842589867995 }, { 296.98821385249204, 421.867555254473 }, { 22.59826114086141, -13.515272641559136 }, { 519.1878710761988, 122.22015547317089 }, { 61.17705180671277, 357.65322649962013 }, { 239.92696170191775, -125.49106433594321 }, { 434.54657780482785, 351.66764702754557 }, { -31.431412441656676, 131.21575125613387 }, { 461.2436494302912, -21.044237866378523 }, { 200.72371071670386, 424.1730997776657 }, { 92.04684925444809, -80.26297299081759 }, { 513.063541838094, 218.3817189215641 }, { 0.7169652182295465, 282.58443517494595 }, { 335.2482626904506, -110.97256255146206 }, { 354.425574093709, 405.36824202151115 }, { -8.61885231898384, 37.46657489501507 }, { 507.7640775141494, 63.52149145210295 }, { 109.26744018275494, 393.2303263402865 }, { 180.41492814031267, -119.23705582626351 }, { 474.2169589406835, 306.8409262097805 }, { -30.40197177037487, 191.08639157435795 }, { 420.02829572653866, -64.50947943435139 }, { 260.49484232681436, 428.3830547196212 }, { 45.1146392479711, -42.979479976492144 }, { 522.4875983679563, 159.16805797555162 }, { 33.77780214061997, 332.62872402543155 }, { 277.0463711812564, -125.54544536666074 }, { 407.2027383454335, 376.7888905531793 }, { -28.23575200529166, 94.2095476342316 }, { 483.89104896410015, 8.412095411284469 }, { 164.114930337209, 417.7476501584265 }, { 123.39234226672954, -100.25942776570716 }, { 503.45337988001984, 254.31153744507168 }, { -16.471565640798417, 249.58791385343835 }, { 370.2224251002356, -98.24727588539832 }, { 320.0291437934352, 419.6132231848885 }, { 7.127963089174955, 3.717787314603015 }, { 518.9523059265587, 99.0556350422276 }, { 77.00469023504533, 374.5788989770183 }, { 216.81551207538533, -127.27864965936655 }, { 452.8002896974706, 337.36804513962727 }, { -35.23109095739056, 154.09764608246326 }, { 448.58319075782805, -40.48835055569688 }, { 223.20109200713222, 429.9584377572644 }, { 71.55924027786594, -69.3403924579824 }, { 520.7929110254843, 196.48094656987206 }, { 9.816074479777853, 303.96047988694124 }, { 314.09201339933594, -120.58943838151751 }, { 376.5280115418159, 398.1643847062561 }, { -20.052226308866125, 57.715856012533806 }, { 502.51287433124287, 40.86045825584013 }, { 128.4540246519516, 406.3952296525152 }, { 157.3675761636545, -115.98107353403017 }, { 489.01509326337293, 288.8648522788693 }, { -29.168597488015223, 214.34461526863612 }, { 403.40149259322743, -80.82995049886586 }, { 283.7866563806503, 429.18426722039123 }, { 27.394490982198874, -27.83058290372776 }, { 525.3198567750662, 136.02041767297257 }, { 47.3312782979184, 351.61504631194686 }, { 254.21979451394444, -130.38969625532545 }, { 427.31189348820203, 364.9363180049287 }, { -35.05755501080779, 116.54039706481078 }, { 473.83200953723224, -12.667795544863196 }, { 185.7789520072908, 426.4974212846338 }, { 101.50141803176805, -92.07297389569783 }, { 514.066750801961, 233.4805696806289 }, { -10.22255157194627, 272.1237733077452 }, { 350.38458725343673, -110.64556665130559 }, { 343.03870526191565, 415.3517915161801 }, { -6.962838578542119, 22.409799779377238 }, { 516.7134858801851, 75.7474386837313 }, { 94.40673801563719, 390.2567618753013 }, { 193.386249685293, -127.08207028581577 }, { 469.94751218977626, 321.3904334217991 }, { -37.080811616821165, 177.46925691524837 }, { 434.1537792267752, -58.975862409330205 }, { 246.33654847483834, 433.84289732361844 }, { 51.870906545039816, -56.57127911720224 }, { 526.6849963543305, 173.75856887931923 }, { 20.82532871806569, 324.70082278290863 }, { 291.95670720393446, -128.44667167183255 }, { 398.16333040056713, 389.0012370838174 }, { -29.8170700798255, 79.09436231729207 }, { 495.2680918480004, 18.494228136123525 }, { 148.91156874240926, 417.995507155818 }, { 134.44028677290808, -110.71235613993721 }, { 502.36452126657775, 269.4856057706225 }, { -25.918627471963475, 237.65861465975917 }, { 385.2498417477046, -95.82878051244438 }, { 307.30984522365407, 427.9803642674908 }, { 10.858727795248683, -11.046638047598265 }, { 526.1737499504063, 112.46737929235493 }, { 62.6176763257113, 369.5633965202647 }, { 230.81669835172485, -133.29741480566324 }, { 446.5375420894379, 351.2660335087886 }, { -39.99943896994188, 139.61468011709377 }, { 461.8842113534686, -33.02553105222927 }, { 208.34783932826264, 433.43825195382885 }, { 80.1656332492299, -81.94101309272125 }, { 522.9560002606719, 211.59002079262495 }, { -1.986028155669345, 294.2759091180021 }, { 329.3403540354573, -121.4177645426195 }, { 365.8390587580561, 409.07592921749733 }, { -19.538053515177637, 42.44599483287732 }, { 512.4486159895739, 52.472159710224346 }, { 113.28067026596531, 404.5411939325609 }, { 169.81317906455516, -124.86322861730028 }, { 485.83419209829844, 303.82430336695563 }, { -36.92745182405457, 201.16049197984552 }, { 418.03118298922004, -76.34545223938528 }, { 269.9651963946683, 435.7587229056313 }, { 33.14921719830292, -42.017091550341576 }, { 530.6579127842692, 150.3730047858124 }, { 33.69805689212632, 344.6333137244286 }, { 268.9930896417931, -134.44885916569487 }, { 419.1559038136633, 377.91031666048207 }, { -37.805176600751565, 101.46017292726285 }, { 486.0458428895364, -3.3994270510580122 }, { 170.50806133975325, 427.9109603299802 }, { 111.811627761995, -103.43147748885161 }, { 514.1339569761429, 248.82427736233012 }, { -20.637080249117673, 260.8503821847635 }, { 365.68258494225296, -109.36458503644829 }, { 330.88831450106, 424.7409109120259 }, { -4.342233906878278, 7.275800950351197 }, { 525.0036088121133, 88.68178447760918 }, { 79.55387412879716, 386.315462240168 }, { 207.00535578851026, -134.20803307367552 }, { 464.71467258784185, 335.8470856537641 }, { -42.98638582059942, 163.26997440536206 }, { 448.10209397073504, -52.490649095838194 }, { 231.6662593301827, 438.48123288400245 }, { 59.55960929313241, -69.90295094819612 }, { 530.0189926477391, 188.786948525715 }, { 8.213944011582214, 315.86686426609384 }, { 307.2273841074191, -130.44927354570578 }, { 388.2512579221244, 400.79415722346334 }, { -30.471486511306694, 63.6991662147002 }, { 506.15060079465485, 29.408875145632564 }, { 133.51070784172455, 417.29535077304126 }, { 146.27414716388876, -120.59945467763978 }, { 500.3138675819798, 284.7731183597268 }, { -34.73292735347553, 224.99608685710677 }, { 400.305753920327, -92.4421463371568 }, { 293.9156877738289, 435.6526935597858 }, { 15.556486662590942, -25.75437338869233 }, { 532.6436954349607, 126.49038067641497 }, { 48.37211595946616, 363.58952542767133 }, { 245.36079897215973, -138.5137971837457 }, { 439.33246691562954, 364.93888251964205 }, { -43.92071713872764, 124.6613559196208 }, { 474.8781319451421, -24.643717901789614 }, { 193.10043745583408, 436.0328531641088 }, { 89.66057221295137, -94.15493248322238 }, { 524.202573537002, 227.01397098617053 }, { -13.324803815028588, 283.74011959256507 }, { 344.82184577733204, -121.30535533756577 }, { 354.3428066397919, 419.45108973301797 }, { -18.066198176324633, 27.026527445025124 }, { 521.7790510591396, 64.84098731370753 }, { 98.04242880426565, 401.71993317946715 }, { 182.95986800833649, -133.07582279184962 }, { 481.68400333185326, 318.7634031583474 }, { -43.95765705032153, 187.33675739517332 }, { 432.55542360624247, -70.89710113319819 }, { 255.5705601172798, 441.5510653888588 }, { 39.854904750594216, -56.01384999581458 }, { 535.1664301942553, 165.22788575452068 }, { 20.337435928227876, 336.72086983330263 }, { 284.1938960704971, -137.63746474434808 }, { 410.0966562388465, 390.5309558447884 }, { -39.647947291221215, 86.03054259174557 }, { 497.82827393349737, 6.737753448451002 }, { 154.96858875316684, 428.3923466320589 }, { 122.94947609952534, -114.28385622597264 }, { 513.2489128355169, 264.3536480202326 }, { -30.474660351421846, 248.7969363001647 }, { 381.08187282869665, -107.11861669185461 }, { 318.01149658751257, 433.486705347521 }, { -0.7513672445585087, -7.8743136551687485 }, { 532.588991955223, 102.28329953793548 }, { 64.77021201086558, 381.4061288343647 }, { 221.2272000945267, -140.57327876244264 }, { 458.52351523460425, 350.14973520094804 }, { -48.08113714528389, 148.53706074538144 }, { 461.8127776328054, -45.06424827023051 }, { 216.5355886364421, 442.26490001550906 }, { 68.16508601401324, -82.91518277251353 }, { 532.461075567107, 204.198883148104 }, { -3.998644250678268, 306.1476413573664 }, { 322.80180578736093, -131.52960711341754 }, { 377.4922845135335, 412.1119911487268 }, { -30.18040591299672, 48.08240262805877 }, { 516.4853910180761, 41.12550094391483 }, { 117.97218888230795, 415.63377599168905 }, { 158.85885149280728, -129.87042168223613 }, { 497.29330617114533, 300.1132959308469 }, { -42.867432519147485, 211.639668602346 }, { 415.3289113396531, -88.08461440042808 }, { 279.88997285078636, 442.5868078964985 }, { 21.21862474887496, -40.34416800969703 }, { 538.3227015501261, 141.07770788539943 }, { 34.328789754634386, 356.6652534722715 }, { 260.3976138966815, -142.89262054577952 }, { 431.19832260080705, 378.326748447476 }, { -46.9642832415563, 109.29075604410785 }, { 487.5065052422149, -15.361357377193599 }, { 177.51444316484123, 437.7163677669023 }, { 100.02056568110802, -105.92554851295321 }, { 524.5117237452337, 242.69519338517205 }, { -24.14593005088352, 272.3814666525883 }, { 360.4772807536111, -120.23642990109565 }, { 342.07225569221237, 429.238240389779 }, { -15.626126002869796, 11.51781069177548 }, { 530.4556463974516, 77.92954973355062 }, { 82.80034219345148, 397.9262396927687 }, { 196.76614885690415, -140.5737879534568 }, { 476.56468493819864, 333.62089371945433 }, { -50.21795574402984, 172.9186889661586 }, { 446.9132111663531, -64.49041869281879 }, { 240.6514598100283, 446.5241642464474 }, { 47.50071192425409, -69.76057825125781 }, { 538.8126901474654, 180.53328834184575 }, { 7.308509100930422, 327.89379235840977 }, { 299.7677798696949, -139.92745633463525 }, { 400.1553400823926, 402.74078197879345 }, { -40.562687408852184, 70.30818301310973 }, { 509.1239303591007, 17.717608690376693 }, { 139.21901839070074, 427.92345478586606 }, { 134.88407947620385, -124.57724416466525 }, { 511.3986856745511, 280.0088399766362 }, { -39.68533808483079, 235.99882745545585 }, { 396.5217117162829, -103.90002013050417 }, { 304.4478788863774, 441.5425165256456 }, { 3.8120538214115527, -22.979374164781603 }, { 539.4266304844014, 116.50842610499316 }, { 50.11688212702646, 375.5318350718515 }, { 236.00462976885677, -146.1387858536552 }, { 451.3824548979932, 364.2377607174477 }, { -52.330326623525764, 133.32090095352794 }, { 475.226183719287, -36.710350548830746 }, { 200.99776592449504, 445.163708541658 }, { 77.66847627267501, -95.54976112534064 }, { 533.9858350007398, 219.93870817292088 }, { -15.756102734565786, 295.567038576228 }, { 338.62230074811504, -131.66723684395785 }, { 365.9151344646508, 422.9007182497911 }, { -28.928523602798407, 32.303306495011284 }, { 526.2211693969505, 53.610762944109055 }, { 102.35635481302114, 413.00072580343283 }, { 172.1566958023534, -138.47708527150874 }, { 493.2981041883953, 315.4451514051808 }, { -50.27746204055535, 197.63113482162257 }, { 430.25815156398255, -82.75681158946256 }, { 265.27822470790437, 448.74186014443126 }, { 27.83916105072231, -54.75515415693704 }, { 543.1740682954605, 156.18043774549108 }, { 20.54780305342399, 348.8018690180769 }, { 275.8751975075145, -146.40161345398113 }, { 422.1516173393643, 391.37075016440673 }, { -49.1025417672131, 93.55744890159909 }, { 499.71212666615156, -5.2000498755612625 }, { 161.64662790789865, 438.46608757902777 }, { 111.21908779365319, -117.19777254576522 }, { 523.8658038492424, 258.5751501208522 }, { -34.39685724615066, 260.2311970547214 }, { 376.2468048218759, -118.19853724960424 }, { 329.0631327518004, 438.3877741013904 }, { -12.210677023586868, -4.019445210348664 }, { 538.4321182973345, 91.69791526624789 }, { 67.61551000461043, 393.15829880280734 }, { 211.18819253011804, -147.31451903592867 }, { 470.47978075522025, 348.3357557774144 }, { -55.66974842819923, 157.95367574783612 }, { 461.0440756720212, -57.134279289556815 }, { 225.25847978554413, 450.64372171632306 }, { 56.072502749261986, -83.19782029250013 }, { 541.5669596279338, 196.23582220502203 }, { -5.3307383877342716, 318.17136950734493 }, { 315.6589554106704, -141.29389358593235 }, { 389.3562468466226, 414.4837261726108 }, { -40.529440328101145, 54.35084600357679 }, { 519.8790902979126, 29.511031253496498 }, { 123.3186240865904, 426.48946462701883 }, { 147.58173891679292, -134.2606790785131 }, { 508.5737050328583, 295.7295131812206 }, { -48.22130222801121, 222.49408655391528 }, { 411.94115928521205, -99.70454747955785 }, { 290.2395313406495, 448.86405133057644 }, { 9.346922607918998, -37.97830451050285 }, { 545.4760894086787, 131.31137909261963 }, { 35.65462479828406, 368.6990330035046 }, { 251.2884969947688, -150.86828780934079 }, { 443.3032387266744, 378.0512244902982 }, { -55.70213442554967, 117.67362735951488 }, { 488.28363776478875, -27.445907216795064 }, { 185.1075069235605, 447.1505293880298 }, { 88.0477555970048, -107.7497256675712 }, { 534.5710405966313, 235.94954539602966 }, { -27.003622881153717, 284.15198365427375 }, { 354.6302658095918, -130.84500140002726 }, { 353.55143117085333, 433.10810281491354 }, { -26.703878570619565, 16.421755262081234 }, { 535.3086743124986, 66.82858575193357 }, { 86.72389689179457, 409.3895308527001 }, { 186.12743055731505, -146.37353626244422 }, { 488.3269345767481, 330.70764637428044 }, { -56.92081170477269, 183.0145992786803 }, { 445.0325583140591, -76.46276272132866 }, { 250.12808175545314, 454.07967507964395 }, { 35.4087504399169, -68.92700597200536 }, { 547.163942451893, 171.747773838886 }, { 7.088427588775943, 340.013965049574 }, { 291.7399862549686, -149.01150445786638 }, { 412.21207918581365, 404.01312561708454 }, { -50.311027551916084, 77.5173530677294 }, { 511.4391877462903, 5.815493725679772 }, { 145.55483511962456, 438.2625412734948 }, { 123.22663538453803, -127.91817853502883 }, { 522.2504855026964, 274.59451781976406 }, { -44.02695226310976, 247.32337889998587 }, { 392.0700709222481, -115.18260161213183 }, { 315.3538084322224, 446.8522396140597 }, { -7.816097249540576, -19.524333591896607 }, { 545.6645618662349, 106.10370625644441 }, { 52.548930183095536, 387.4177065928094 }, { 226.17994057663148, -153.25799468820273 }, { 463.436224097969, 362.84736910135234 }, { -60.277204587086544, 142.4911024773413 }, { 474.8882406490535, -48.84089960163692 }, { 209.44395101931926, 453.87837316598575 }, { 65.55287166370962, -96.26710291409356 }, { 543.402580576936, 212.28061279151376 }, { -17.523586051461507, 307.57606136493337 }, { 331.81042674209453, -141.7150294991512 }, { 377.72671793326066, 425.70525659386874 }, { -39.53153682181551, 38.21721046480718 }, { 530.0418296408502, 42.086009135947734 }, { 107.32731634570814, 424.07891236057503 }, { 161.00601842407548, -143.28524356567485 }, { 504.7678001777322, 311.4549863012189 }, { -56.037017919534264, 208.3232929937488 }, { 427.2792300356466, -94.53136799783323 }, { 275.4308643251362, 455.40950723645426 }, { 15.848723993685383, -52.81028398406144 }, { 550.6996196079951, 146.64425899974776 }, { 21.443628290009514, 360.9175483836402 }, { 267.0277817911485, -154.72837879459283 }, { 434.30092705211905, 391.5310334462205 }, { -58.16775446402784, 101.64898781829652 }, { 500.92759617824737, -17.291096888098593 }, { 168.9208738160932, 448.2013772913201 }, { 99.27778272699919, -119.45952462972812 }, { 534.1977119090279, 252.17344961331807 }, { -37.68807173156273, 271.9323867185278 }, { 370.7663184944254, -129.04907065224043 }, { 340.4356231970507, 442.6838390530571 }, { -23.497896891738606, 0.49811191667839694 }, { 543.7008137290118, 80.74024766982521 }, { 71.13569435028515, 404.79693888593806 }, { 200.72835906679083, -153.51625822192202 }, { 482.3818912980287, 345.839853560944 }, { -62.7578750081845, 167.836404978642 }, { 459.5916247990387, -69.20989140922993 }, { 234.4891766227666, 458.56486036619094 }, { 43.914697193986484, -82.80010241345673 }, { 550.2614176635869, 187.72717586885778 }, { -5.991059764713356, 330.3194113233741 }, { 307.9369364603926, -150.69611048084326 }, { 401.4026140794067, 416.1973901457129 }, { -50.56848176199486, 61.22759209860007 }, { 522.6334302720853, 17.65750787012712 }, { 129.29782876730437, 437.0895573450372 }, { 136.01079719976104, -138.0351518027696 }, { 519.6548084845541, 290.6933441517507 }, { -52.98764068294537, 233.69481951337988 }, { 407.88639951576664, -111.18295792741989 }, { 300.9852027172423, 454.58647608904016 }, { -2.442059932123641, -34.93591441881429 }, { 552.1115768918507, 121.10220507841659 }, { 37.661334573371846, 380.7094767276577 }, { 241.69322192623514, -158.36689342953943 }, { 455.4443300798998, 377.09567708839995 }, { -64.00736819127798, 126.58222293934712 }, { 488.3867865645974, -39.62581646238826 }, { 193.26181559988788, 456.19978114905985 }, { 75.92117999152478, -108.91109670744387 }, { 544.2960517304261, 228.6114447662245 }, { -29.214735403022814, 296.1334493484012 }, { 348.1641377938326, -141.1723793883406 }, { 365.2970804001286, 436.3525308281088 }, { -37.55565058936787, 21.966726562740718 }, { 539.5621682837998, 55.407703221267894 }, { 91.30548209783214, 420.6837322016511 }, { 175.11783508832397, -151.60420403355792 }, { 499.9782275104616, 327.12440015068137 }, { -63.0893572900988, 193.52947259370413 }, { 442.4750606367844, -88.38308101114606 }, { 260.06851537973705, 461.13969355887514 }, { 23.309536079450197, -67.41491322517302 }, { 555.062268746999, 162.45717550868676 }, { 7.543363472029853, 352.2005644029994 }, { 283.16972499994256, -157.68861337557396 }, { 424.3938626169478, 404.6191025586636 }, { -59.70148208399627, 85.30220435026874 }, { 513.1018064456441, -6.26928044428854 }, { 152.4951265890411, 448.2954857809659 }, { 111.33035863920068, -130.62517056185695 }, { 532.8501800692244, 268.5515634166961 }, { -47.758142008227935, 258.9410677391843 }, { 386.97045678006134, -126.26899353903607 }, { 326.6048987387036, 451.5796943329252 }, { -19.305425382292356, -15.40693848962323 }, { 551.3528004684247, 95.30447857986529 }, { 55.65264845474502, 399.2231339188669 }, { 215.91444697146474, -159.86425371477108 }, { 475.4684938401548, 360.7811239375291 }, { -67.75175938448007, 152.14500390718732 }, { 473.8754207196959, -61.00900985166206 }, { 218.41300603954858, 462.16491184682843 }, { 53.34097986915941, -96.3156928496035 }, { 552.4386279576968, 204.064498642773 }, { -18.63353988692006, 319.73931500305787 }, { 324.40967109969966, -151.43241782549924 }, { 389.7492522694275, 427.8684954163042 }, { -49.856919737234875, 44.74634107264875 }, { 533.2423000232031, 30.29523971332425 }, { 112.93513439749103, 434.93431824524646 }, { 149.5363346033819, -147.49903636789938 }, { 516.0712206852976, 306.8112110549358 }, { -61.23254658035313, 219.38497202792715 }, { 423.634944822393, -106.19737734970582 }, { 286.0006795394652, 461.5477442288069 }, { 3.9083236312631016, -50.1933822597135 }, { 557.7343892906008, 136.6464707638661 }, { 23.0130205214719, 373.04203645350043 }, { 257.6778798373618, -162.60670446462152 }, { 446.51777681514443, 391.0213542873543 }, { -66.8302570048761, 110.28002366696585 }, { 501.48181617769035, -29.507853388690677 }, { 176.76748212013337, 457.5827224180457 }, { 87.15360385037641, -121.07377795406595 }, { 544.2271026450694, 245.17091382189454 }, { -40.35046713105794, 283.8721742128654 }, { 364.6611302237742, -139.65078132744546 }, { 352.1005713394792, 446.3745471147686 }, { -34.59184463604822, 5.65945305801182 }, { 548.3922142428121, 69.43853596149337 }, { 75.3138184580427, 416.29928830773406 }, { 189.87556016408087, -159.1731465668304 }, { 494.205687783314, 342.67688621797544 }, { -69.33772601934629, 178.15798488733412 }, { 457.46807944065324, -81.26571825940465 }, { 244.2012257306413, 466.0181477072142 }, { 31.71804275029632, -81.73238341061915 }, { 558.5319863080972, 178.69838079645427 }, { -5.987583735475738, 342.5645942929553 }, { 299.6599704120309, -159.72159950421988 }, { 413.60362853665976, 417.25851946758337 }, { -60.280794262377356, 68.6898232885043 }, { 524.7514675371087, 5.593055341572978 }, { 135.88856660314735, 447.41537395472255 }, { 124.17429684437099, -141.19439528054292 }, { 530.5161406015786, 285.02427900128845 }, { -57.164500395064636, 245.21367283487996 }, { 403.1822250377743, -122.49773649493582 }, { 312.09908915906584, 459.74964963956256 }, { -14.124755322460771, -31.23255763362542 }, { 558.2222837457205, 110.47756844583853 }, { 40.33551216203119, 392.67174505074263 }, { 231.63844196204852, -165.37916588800377 }, { 467.59568103382094, 375.4712573056161 }, { -71.8683968834668, 135.99082700656436 }, { 487.8247617644598, -51.87429767887687 }, { 201.9527914981459, 464.8503124453689 }, { 63.668287280817, -109.416064158316 }, { 553.6708340892831, 220.70413965836764 }, { -30.783214820022238, 308.29797011008156 }, { 341.10063450778705, -151.20065527631752 }, { 377.28108356884195, 438.9729878677326 }, { -48.16169026827231, 28.13266595502897 }, { 543.2150990237899, 43.69502758461498 }, { 96.52687380353632, 431.78740535779855 }, { 163.76527316636242, -156.26227982206566 }, { 511.4956084132431, 322.88740349278453 }, { -68.7176288909572, 204.43583135970206 }, { 439.2548657023118, -100.22708263243089 }, { 270.44593110718347, 467.69585307046395 }, { 11.22850377462467, -65.23623797300505 }, { 562.4969673342073, 152.68746544418332 }, { 8.663679708307114, 364.4272118370643 }, { 274.0819081449767, -165.94583243103133 }, { 436.6735756690773, 404.56597572103624 }, { -68.71895502928402, 93.63907892876477 }, { 514.1166207060514, -18.509076053398474 }, { 160.01767300882693, 458.0051673340005 }, { 99.2231931322053, -132.70059034837928 }, { 543.1787594433487, 261.90058583285065 }, { -50.87879712326077, 270.82386305523175 }, { 381.24170783510226, -137.13844770530645 }, { 338.17325141264405, 455.7222934577122 }, { -30.633608221543852, -10.644111110882477 }, { 556.4863046909388, 84.13829066060596 }, { 59.41316160673131, 410.92439681669896 }, { 205.23513042063286, -165.95010879057597 }, { 487.4543330380948, 358.05173917007926 }, { -74.74418499497955, 162.25640055077298 }, { 472.19817903958096, -73.18873566325516 }, { 227.8797074294097, 470.0112456870405 }, { 41.059557036144184, -95.70364753178376 }, { 561.0797220696381, 195.31441167068323 }, { -19.091681034796864, 332.0294429996675 }, { 316.44271509151497, -160.80308419661085 }, { 401.9549952861154, 429.3937092413544 }, { -59.88642180901422, 51.86955791726287 } };
	final static private int[] samples =
			new int[]
					{ 69, 205, 175, 222, 207, 145, 220, 85, 0, 134, 0, 13, 185, 169, 170, 134, 212, 220, 153, 0, 255, 145, 208, 255, 160, 242, 217, 248, 28, 255, 113, 247, 255, 181, 136, 221, 255, 167, 255, 157, 166, 226, 113, 204, 199, 162, 248, 112, 0, 241, 222, 221, 222, 255, 205, 127, 213, 0, 179, 119, 66, 93, 253, 255, 167, 194, 175, 155, 154, 177, 170, 231, 134, 162, 170, 0, 174, 255, 168, 0, 253, 42, 0, 208, 209, 108, 255, 168, 250, 175, 0, 167, 255, 229, 179, 255, 215, 231, 208, 182, 175, 255, 128, 0, 189, 204, 13, 192, 187, 192, 13, 109, 255, 193, 155, 210, 231, 167, 182, 175, 148, 149, 170, 184, 34, 167, 98, 252, 0, 180, 147, 0, 203, 185, 180, 236, 215, 169, 120, 145, 195, 219, 210, 173, 56, 128, 230, 155, 172, 0, 254, 75, 131, 255, 186, 225, 0, 215, 217, 197, 255, 197, 99, 249, 241, 147, 0, 255, 245, 224, 0, 187, 235, 59, 221, 210, 195, 178, 165, 135, 195, 197, 50, 252, 229, 195, 220, 142, 149, 221, 165, 148, 209, 223, 206, 251, 147, 212, 199, 113, 202, 178, 143, 210, 178, 46, 218, 237, 254, 92, 166, 151, 13, 154, 133, 219, 175, 228, 0, 0, 149, 50, 255, 208, 167, 185, 222, 219, 102, 99, 150, 182, 187, 136, 0, 133, 0, 203, 0, 144, 159, 172, 166, 231, 255, 115, 178, 179, 226, 215, 215, 0, 13, 202, 233, 0, 169, 208, 0, 255, 88, 251, 255, 179, 199, 180, 218, 146, 141, 142, 142, 120, 193, 231, 229, 255, 170, 255, 223, 173, 182, 192, 189, 69, 214, 178, 248, 133, 177, 178, 209, 226, 215, 159, 0, 178, 250, 171, 154, 253, 184, 214, 130, 128, 231, 223, 0, 206, 169, 138, 253, 230, 215, 176, 131, 199, 189, 197, 211, 108, 138, 224, 255, 118, 255, 175, 243, 140, 0, 213, 220, 145, 223, 131, 147, 188, 212, 178, 140, 218, 139, 34, 222, 131, 180, 219, 162, 189, 34, 127, 255, 254, 13, 108, 145, 177, 157, 233, 168, 230, 255, 96, 210, 201, 175, 202, 217, 13, 207, 0, 175, 219, 0, 0, 222, 196, 205, 120, 99, 196, 255, 169, 223, 0, 0, 185, 253, 255, 130, 205, 243, 199, 175, 255, 22, 157, 217, 202, 117, 228, 34, 114, 190, 224, 208, 221, 122, 231, 255, 220, 213, 247, 216, 190, 192, 231, 64, 137, 222, 156, 140, 212, 13, 198, 167, 79, 216, 200, 119, 191, 134, 171, 223, 146, 155, 223, 202, 149, 253, 71, 0, 255, 255, 167, 254, 227, 0, 255, 155, 228, 128, 0, 230, 42, 255, 28, 175, 193, 194, 230, 255, 202, 173, 219, 224, 213, 0, 46, 227, 206, 202, 222, 0, 220, 187, 212, 196, 0, 223, 179, 0, 197, 138, 220, 214, 255, 192, 166, 224, 255, 0, 204, 135, 202, 221, 223, 170, 0, 185, 166, 117, 169, 13, 77, 189, 215, 159, 202, 178, 226, 220, 179, 180, 161, 255, 206, 206, 114, 218, 129, 186, 137, 182, 138, 250, 56, 0, 182, 192, 167, 90, 0, 177, 248, 205, 218, 0, 221, 0, 153, 142, 185, 238, 196, 255, 241, 223, 161, 194, 149, 120, 252, 71, 0, 71, 248, 167, 181, 115, 211, 42, 166, 0, 165, 147, 95, 203, 209, 193, 124, 201, 118, 220, 0, 228, 153, 247, 255, 170, 222, 105, 177, 0, 179, 197, 148, 0, 201, 59, 167, 217, 255, 136, 192, 140, 168, 255, 0, 150, 202, 178, 244, 226, 93, 66, 223, 197, 136, 160, 185, 173, 196, 0, 85, 174, 252, 204, 175, 208, 182, 170, 0, 121, 185, 218, 198, 240, 148, 250, 0, 198, 218, 177, 28, 150, 161, 205, 169, 173, 238, 148, 252, 255, 254, 179, 255, 219, 249, 157, 219, 199, 0, 176, 230, 77, 156, 185, 155, 218, 22, 160, 232, 255, 229, 178, 0, 233, 190, 95, 160, 145, 152, 185, 0, 224, 143, 255, 0, 162, 215, 213, 148, 0, 0, 223, 181, 95, 0, 151, 162, 185, 221, 171, 201, 242, 142, 121, 237, 0, 0, 254, 223, 218, 0, 0, 242, 147, 195, 127, 0, 255, 150, 140, 230, 255, 203, 95, 99, 250, 184, 255, 129, 255, 247, 69, 158, 215, 197, 227, 136, 146, 181, 216, 90, 205, 187, 184, 170, 144, 170, 139, 183, 150, 255, 98, 164, 215, 189, 141, 190, 202, 118, 108, 145, 183, 226, 134, 203, 96, 246, 229, 90, 208, 180, 0, 167, 255, 214, 255, 221, 128, 219, 255, 71, 255, 248, 0, 64, 151, 179, 171, 117, 230, 255, 199, 98, 255, 81, 118, 193, 93, 255, 255, 241, 210, 137, 209, 176, 182, 175, 0, 232, 167, 112, 255, 170, 224, 228, 0, 255, 253, 249, 171, 169, 0, 211, 181, 181, 255, 135, 173, 229, 223, 109, 235, 194, 255, 185, 255, 224, 130, 224, 13, 218, 0, 194, 254, 132, 159, 0, 230, 0, 188, 255, 251, 183, 255, 0, 143, 234, 192, 255, 135, 155, 195, 255, 237, 246, 255, 117, 237, 176, 240, 172, 185, 200, 196, 176, 194, 203, 129, 180, 0, 255, 255, 185, 135, 153, 195, 232, 0, 188, 133, 173, 148, 0, 64, 134, 158, 251, 220, 244, 224, 0, 227, 209, 121, 0, 0, 231, 114, 201, 142, 193, 0, 255, 159, 190, 148, 221, 255, 232, 255, 164, 93, 231, 163, 209, 79, 171, 206, 146, 150, 255, 210, 187, 204, 222, 182, 192, 255, 221, 230, 102, 101, 222, 173, 255, 192, 156, 212, 96, 221, 218, 188, 187, 0, 0, 117, 222, 13, 204, 188, 71, 228, 56, 216, 255, 216, 122, 0, 180, 202, 237, 254, 192, 132, 161, 120, 228, 0, 202, 209, 124, 151, 255, 131, 244, 151, 218, 218, 0, 13, 161, 202, 194, 181, 164, 0, 251, 169, 210, 251, 196, 200, 124, 73, 114, 171, 187 };
//					{ 57, 42, 31, 43, 34, 33, 42, 29, 40, 39, 30, 45, 36, 35, 46, 33, 48, 52, 35, 54, 41, 39, 53, 41, 56, 45, 39, 62, 62, 48, 48, 66, 46, 60, 61, 47, 64, 48, 56, 64, 46, 59, 54, 48, 70, 40, 59, 61, 46, 75, 47, 58, 68, 50, 69, 60, 53, 82, 57, 62, 71, 51, 76, 53, 50, 68, 57, 62, 65, 48, 90, 54, 62, 80, 55, 79, 54, 49, 91, 48, 66, 79, 47, 93, 53, 56, 92, 50, 92, 70, 48, 105, 58, 84, 101, 45, 102, 72, 59, 97, 47, 80, 89, 47, 98, 62, 78, 109, 51, 101, 89, 64, 105, 66, 90, 93, 61, 116, 83, 77, 103, 58, 87, 92, 62, 126, 78, 85, 126, 78, 106, 113, 70, 111, 72, 90, 97, 75, 118, 85, 92, 108, 86, 116, 100, 94, 127, 99, 110, 116, 99, 130, 109, 109, 116, 72, 106, 118, 88, 124, 99, 103, 131, 104, 118, 109, 76, 124, 117, 108, 115, 103, 117, 103, 103, 103, 108, 125, 135, 104, 127, 104, 119, 134, 105, 130, 132, 124, 127, 98, 107, 115, 103, 134, 77, 113, 120, 113, 128, 109, 105, 127, 120, 108, 123, 116, 131, 113, 111, 128, 108, 120, 131, 103, 127, 119, 104, 140, 112, 128, 128, 114, 128, 117, 117, 129, 107, 110, 109, 101, 30, 116, 135, 146, 112, 70, 121, 116, 142, 118, 130, 140, 129, 134, 110, 127, 114, 115, 131, 85, 125, 132, 121, 128, 120, 112, 119, 126, 117, 136, 120, 114, 128, 110, 116, 118, 124, 148, 110, 134, 129, 129, 149, 107, 90, 147, 132, 135, 103, 121, 109, 116, 21, 79, 117, 60, 121, 30, 132, 124, 21, 131, 115, 141, 125, 141, 140, 132, 79, 123, 127, 111, 120, 149, 94, 130, 29, 124, 136, 119, 111, 100, 134, 132, 103, 108, 93, 116, 127, 19, 120, 111, 154, 122, 114, 131, 133, 147, 127, 140, 155, 138, 135, 114, 98, 38, 121, 18, 62, 126, 134, 128, 72, 118, 121, 22, 131, 121, 141, 123, 111, 144, 98, 133, 126, 131, 70, 127, 149, 99, 83, 65, 109, 46, 92, 117, 118, 93, 128, 52, 129, 22, 78, 124, 16, 89, 14, 154, 135, 22, 125, 129, 127, 127, 143, 141, 122, 17, 118, 92, 26, 123, 12, 70, 131, 34, 117, 119, 113, 127, 47, 118, 125, 68, 70, 16, 127, 113, 45, 126, 133, 59, 128, 20, 122, 48, 26, 119, 149, 48, 138, 112, 66, 86, 28, 123, 12, 68, 67, 126, 67, 13, 112, 107, 13, 118, 131, 96, 106, 38, 137, 36, 56, 129, 133, 60, 125, 0, 62, 26, 59, 120, 26, 93, 124, 112, 91, 124, 28, 95, 15, 63, 135, 14, 66, 18, 23, 134, 0, 111, 86, 29, 123, 86, 63, 65, 22, 68, 21, 27, 119, 0, 63, 78, 47, 119, 108, 85, 64, 0, 90, 14, 45, 51, 10, 136, 34, 120, 111, 123, 23, 121, 0, 46, 16, 26, 133, 57, 61, 129, 0, 55, 24, 33, 134, 0, 62, 22, 63, 73, 42, 19, 42, 0, 101, 105, 35, 77, 13, 120, 40, 13, 95, 66, 27, 126, 0, 56, 38, 21, 94, 12, 123, 130, 0, 93, 104, 30, 62, 0, 83, 119, 15, 88, 57, 24, 128, 0, 50, 19, 29, 122, 20, 49, 90, 0, 76, 9, 36, 84, 0, 68, 64, 34, 86, 11, 22, 37, 0, 85, 82, 65, 76, 0, 152, 40, 0, 103, 38, 29, 107, 0, 52, 25, 23, 105, 10, 118, 117, 0, 89, 9, 35, 113, 0, 60, 33, 14, 96, 10, 23, 35, 0, 85, 25, 31, 85, 0, 44, 86, 0, 99, 9, 40, 107, 0, 61, 28, 17, 96, 9, 32, 96, 0, 78, 34, 32, 88, 0, 110, 120, 0, 95, 14, 33, 73, 0, 100, 32, 26, 126, 0, 51, 45, 0, 68, 9, 40, 104, 0, 66, 33, 33, 88, 10, 29, 36, 0, 69, 27, 31, 95, 0, 121, 81, 0, 109, 15, 44, 115, 0, 78, 35, 19, 50, 8, 41, 78, 0, 83, 15, 35, 88, 0, 56, 78, 0, 91, 10, 35, 44, 0, 69, 38, 31, 95, 0, 40, 35, 0, 64, 9, 46, 93, 0, 61, 38, 13, 79, 8, 37, 43, 0, 76, 28, 31, 88, 0, 165, 96, 0, 116, 10, 43, 51, 0, 75, 40, 22, 74, 0, 43, 83, 0, 102, 18, 38, 111, 0, 63, 50, 0, 67, 9, 36, 36, 0, 79, 43, 31, 88, 0, 28, 52, 0, 62, 11, 49, 92, 0, 118, 42, 15, 62, 0, 42, 110, 0, 84, 30, 32, 73, 0, 88, 71, 0, 83, 10, 41, 60, 0, 77, 46, 29, 83, 0, 41, 33, 0, 107, 19, 45, 74, 0, 73, 41, 0, 51, 8, 39, 42, 0, 82, 40, 30, 69, 0, 152, 70, 0, 54, 11, 48, 78, 0, 73, 45, 20, 42, 0, 42, 45, 0, 93, 29, 36, 63, 0, 93, 77, 0, 43, 9, 40, 41, 0, 86, 44, 33, 64, 0, 31, 33, 0, 89, 16, 49, 79, 0, 83, 39, 12, 47, 0, 41, 98, 0, 76, 33, 28, 58, 0, 101, 74, 0, 54, 10, 45, 73, 0, 85, 47, 26, 71, 0, 46, 40, 0, 92, 22, 41, 60, 0, 118, 37, 0, 40, 8, 36, 38, 0, 81, 35, 34, 50, 0, 30, 37, 0, 64, 13, 48, 87, 0, 57, 40, 16, 50, 0, 41, 48, 0, 74, 27, 26, 54, 0, 91, 69, 0, 58, 8, 40, 58, 0, 89, 42, 33, 44, 0, 50, 34, 0, 75, 17, 46, 65, 0, 66, 34, 0, 43, 0, 36, 51, 0, 68, 26, 32, 52, 0, 177, 59, 0, 46, 9, 43, 90, 0, 91, 36, 21, 59, 0, 49, 43, 0, 74, 20, 31, 54, 0, 45, 68, 0, 47, 0, 32, 40, 0, 78, 28, 38, 43, 0, 82, 37, 0, 61, 12, 44, 72, 0, 47, 31, 0, 57, 0, 40, 47, 0, 61, 17, 26, 54, 0, 72, 68, 0, 39, 8, 36, 76, 0, 89, 32, 30, 36, 0, 59, 39, 0, 66, 15, 39, 60, 0, 44, 34, 0, 43, 0, 27, 67, 0, 65, 17, 36, 49, 0, 125, 45, 0, 53, 8, 40, 82, 0, 83, 26, 0, 45, 0, 51, 54, 0, 59, 12, 26, 60, 0, 55, 68, 0, 45, 0, 28, 49, 0, 75, 22, 36, 43, 0, 62, 46, 0, 68, 10, 38, 59, 0, 37, 28, 0, 46, 0, 42, 76, 0, 58, 10, 31, 48, 0, 113, 55, 0, 44, 8, 31, 112, 0, 88, 22, 23, 44, 0, 66, 47, 0, 65, 10, 32, 68, 0, 27, 46, 0, 56, 0, 23, 54, 0, 67, 13, 37, 46, 0, 64, 53, 0, 61, 8, 38, 59, 0, 46, 22, 0, 53, 0, 57, 53, 0, 55, 9, 26, 50, 0, 70, 64, 0, 36, 0, 26, 53, 0, 76, 17, 30, 42, 0, 75, 52, 0, 75, 9, 36, 63, 0, 28, 0, 0, 0, 0, 43, 67, 0, 60, 9, 33, 40, 0, 170, 55, 0, 47, 8, 32, 75, 0, 87, 0, 0, 0, 0, 73, 64, 0, 66, 8, 30, 60, 0, 24, 0, 0, 0, 0, 26, 49, 0, 68, 0, 33, 48, 0, 72, 58, 0, 65, 9, 37, 51, 0, 50, 0, 0, 0, 0, 63, 92, 0, 59, 8, 28, 37, 0, 97, 57, 0, 0, 0, 29, 102, 0, 76, 0, 0, 0, 0, 82, 53, 0, 76, 9, 35, 64, 0, 30, 0, 0, 0, 0, 43, 75, 0, 65, 0, 32, 0, 0, 76, 59, 0, 0, 9, 35, 51, 0, 70, 0, 0, 0, 0, 82, 77, 0, 71, 9, 29, 48, 0, 53, 0, 0, 0, 0, 33, 62, 0, 70, 0, 25, 0, 0, 80, 50, 0, 0, 10, 37, 54, 0, 19, 0, 0, 0, 0, 69, 101, 0, 68, 0, 30, 0, 0, 135, 0, 0, 0, 0, 30, 58, 0, 75, 0, 0, 0, 0, 88, 57, 0, 76, 10, 35, 58, 0, 55, 0, 0, 0, 0, 40, 73, 0, 69, 0, 27, 0, 0, 66, 0, 0, 0, 12, 37, 48, 0, 23, 0, 0, 0, 0, 89, 78, 0, 81, 0, 31, 0, 0, 107, 0, 0, 0, 0, 36, 83, 0, 69, 0, 0, 0, 0, 83, 44, 0, 0, 11, 39, 54, 0, 18, 0, 0, 0, 0, 65, 95, 0, 78, 0, 29, 0, 0, 103, 0, 0, 0, 11, 32, 56, 0, 74, 0, 0, 0, 0, 94, 79, 0, 0, 0, 38, 0, 0, 95, 0, 0, 0, 0, 39, 49, 0, 73, 0, 0, 0, 0, 71, 0, 0, 0, 12, 39, 47, 0, 32, 0, 0, 0, 0, 89, 38, 0, 92, 0, 34, 0, 0, 94, 0, 0, 0, 0, 34, 81, 0, 71, 0, 0, 0, 0, 89, 47, 0, 0, 0, 43, 51, 0, 69, 0, 0, 0, 0, 55, 50, 0, 88, 0, 27, 0, 0, 67, 0, 0, 0, 12, 35, 61, 0, 59, 0, 0, 0, 0, 101, 84, 0, 0, 0, 43, 0, 0, 90, 0, 0, 0, 0, 38, 58, 0, 77, 0, 0, 0, 0, 76, 0, 0, 0, 0, 42, 44, 0, 63, 0, 0, 0, 0, 78, 25, 0, 101, 0, 31, 0, 0, 134, 0, 0, 0, 0, 31, 79, 0, 75, 0, 0, 0, 0, 92, 0, 0, 0, 0, 47, 0, 0, 84, 0, 0, 0, 0, 43, 29, 0, 93, 0, 0, 0, 0, 60, 0, 0, 0, 0, 40, 79, 0, 75, 0, 0, 0, 0, 97, 51, 0, 0, 0, 42, 0, 0, 66, 0, 0, 0, 0, 35, 44, 0, 80, 0, 0, 0, 0, 80, 0, 0, 0, 0, 47, 0, 0, 86, 0, 0, 0, 0, 64, 23, 0, 107, 0, 29, 0, 0, 60, 0, 0, 0, 0, 33, 64, 0, 85, 0, 0, 0, 0, 97, 0, 0, 0, 0, 50, 0, 0, 51, 0, 0, 0, 0, 36, 32, 0, 93, 0, 0, 0, 0, 59, 0, 0, 0, 0, 42, 0, 0, 100, 0, 0, 0, 0, 86, 32, 0, 0, 0, 38, 0, 0, 59, 0, 0, 0, 0, 28, 64, 0, 86, 0, 0, 0, 0, 83, 0, 0, 0, 0, 52, 0, 0, 74, 0, 0, 0, 0, 43, 21, 0, 107, 0, 0, 0, 0, 47, 0, 0, 0, 0, 37, 61, 0, 81, 0, 0, 0, 0, 99, 0, 0, 0, 0, 51, 0, 0, 46, 0, 0, 0, 0, 28, 61, 0, 89, 0, 0, 0, 0, 62, 0, 0, 0, 0, 46, 0, 0, 95, 0, 0, 0, 0, 69, 17, 0, 0, 0, 32, 0, 0, 75, 0, 0, 0, 0, 27, 60, 0, 94, 0, 0, 0, 0, 90, 0, 0, 0, 0, 54, 0, 0, 56, 0, 0, 0, 0, 30, 22, 0, 96, 0, 0, 0, 0, 42, 0, 0, 0, 0, 40, 0, 0, 78, 0, 0, 0, 0, 92, 0, 0, 0, 0, 45, 0, 0, 43, 0, 0, 0, 0, 22, 62, 0, 90, 0, 0, 0, 0, 70, 0, 0, 0, 0, 49, 0, 0, 58, 0, 0, 0, 0, 42, 18, 0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0, 32, 0, 0, 96, 0, 0, 0, 0, 96, 0, 0, 0, 0, 55, 0, 0, 49, 0, 0, 0, 0, 22, 57, 0, 81, 0, 0, 0, 0, 43, 0, 0, 0, 0, 41, 0, 0, 75, 0, 0, 0, 0, 77, 0, 0, 0, 0, 34, 0, 0, 38, 0, 0, 0, 0, 24, 54, 0, 97, 0, 0, 0, 0, 79, 0, 0, 0, 0, 50, 0, 0, 54, 0, 0, 0, 0, 22, 49, 0, 0, 0, 0, 0, 0, 27, 0, 0, 0, 0, 36, 0, 0, 94, 0, 0, 0, 0, 95, 0, 0, 0, 0 };
	
	private KNearestNeighborSearchBehavior(){}
	
	final static private < T extends Type< T > > long drawNearestNeighbor(
			final IterableInterval< T > target,
			final NearestNeighborSearch< T > nnSearch )
	{
		final Timer timer = new Timer();
		timer.start();
		final Cursor< T > c = target.localizingCursor();
		while ( c.hasNext() )
		{
			c.fwd();
			nnSearch.search( c );
			c.get().set( nnSearch.getSampler().get() );
		}
		return timer.stop();
	}

	final static private < T extends RealType< T > > long drawWeightedByDistance(
			final IterableInterval< T > target,
			final KNearestNeighborSearch< T > knnSearch,
			final int k,
			final double p,
			final double min,
			final double max )
	{
		final Timer timer = new Timer();
		timer.start();
		final Cursor< T > c = target.localizingCursor();
		while ( c.hasNext() )
		{
			c.fwd();
			knnSearch.search( c );
			double s = 0;
			double v = 0;
			for ( int i = 0; i < k; ++i )
			{
				final double d = knnSearch.getSquareDistance( i );
				if ( d > 0.001 )
				{
					final double w = 1.0 / Math.pow(  d, p );
					v += w * knnSearch.getSampler( i ).get().getRealDouble();
					s += w;
				}
				else
				{
					s = 1.0;
					v = knnSearch.getSampler( i ).get().getRealDouble();
				}
			}
			v /= s;
			
			c.get().setReal( Math.max(  min, Math.min( max, v ) ) );
		}
		return timer.stop();
	}
	
	final static private < T extends RealType< T > > long drawWeightedByDistanceInterpolator(
			final IterableInterval< T > target,
			final KNearestNeighborSearch< T > knnSearch,
			final int k,
			final double p,
			final double min,
			final double max )
	{
		final Timer timer = new Timer();
		timer.start();
		final Cursor< T > c = target.localizingCursor();
		final InverseDistanceWeightingInterpolator< T > interpolator= new InverseDistanceWeightingInterpolatorFactory< T >( p ).create( knnSearch, target );
		while ( c.hasNext() )
		{
			c.fwd();
			interpolator.setPosition( c );
			c.get().setReal( Math.max(  min, Math.min( max, interpolator.get().getRealDouble() ) ) );
		}
		return timer.stop();
	}
	
	
	
	
	
	final static public void main( final String[] args )
	{
		final RealPointSampleList< UnsignedByteType > list = new RealPointSampleList< UnsignedByteType >( 2 );
		for ( int i = 0; i < samples.length; ++i )
			list.add( new RealPoint( coordinates[ i ] ), new UnsignedByteType( samples[ i ] ) );
		
		final KDTree< UnsignedByteType > kdtree = new KDTree< UnsignedByteType >( list );
		
		final ImagePlusImgFactory< UnsignedByteType > factory = new ImagePlusImgFactory< UnsignedByteType >();
		
		new ImageJ();
		
		IJ.log( "Linear Search" );
		IJ.log( "=============" );
		
		/* nearest neighbor */
		IJ.log( "Nearest neighbor ..." );
		final ImagePlusImg< UnsignedByteType, ? > img1 = factory.create( size, new UnsignedByteType() );
		long t = drawNearestNeighbor(
				img1,
				new NearestNeighborSearchOnIterableRealInterval< UnsignedByteType >( list ) );
		
		IJ.log( t + "ms " );
		
		try
		{
			img1.getImagePlus().show();
			IJ.log( "Done." );
		}
		catch ( final ImgLibException e )
		{
			IJ.log( "Didn't work out." );
			e.printStackTrace();
		}
		
		/* nearest neighbor through k-nearest-neighbor search */
		IJ.log( "Nearest neighbor through k-nearest-neighbor search ..."  );
		final ImagePlusImg< UnsignedByteType, ? > img2 = factory.create( size, new UnsignedByteType() );
		t = drawNearestNeighbor(
				img2,
				new KNearestNeighborSearchOnIterableRealInterval< UnsignedByteType >( list, 1 ) );
		
		IJ.log( t + "ms " );
		
		try
		{
			img2.getImagePlus().show();
			IJ.log( "Done." );
		}
		catch ( final ImgLibException e )
		{
			IJ.log( "Didn't work out." );
			e.printStackTrace();
		}
		
		/* weighted by distance */
		IJ.log( "Weighted by distance ..." );
		final ImageStack distanceStack = new ImageStack( ( int )size[ 0 ], ( int )size[ 1 ] );
		
		int np = 1;
		for ( int k = 5; k <= 30; k += 5 )
		{
			np = 0;
			t = 0;
			for ( double p = 0; p <= 2; p += 0.5, ++np )
			{
				IJ.log( "  k=" + k + " p=" + String.format( "%.2f", p ) );
				final ImagePlusImg< UnsignedByteType, ? > img3 = factory.create( size, new UnsignedByteType() );
				t += drawWeightedByDistanceInterpolator( img3, new KNearestNeighborSearchOnIterableRealInterval< UnsignedByteType >( list, k ), k, p, 0, 255 );
				try
				{
					distanceStack.addSlice( "k=" + k + " p=" + String.format( "%.2f", p ), img3.getImagePlus().getProcessor() );
				}
				catch ( final ImgLibException e )
				{
					IJ.log( "Didn't work out." );
					e.printStackTrace();
				}
			}
			
			IJ.log( t + "ms" );
		}
		
		final ImagePlus impDistance = new ImagePlus( "weighted by distance", distanceStack );
		impDistance.setOpenAsHyperStack( true );
		impDistance.setDimensions( 1, np, distanceStack.getSize() / np );
		impDistance.show();
		IJ.log( "Done." );
		
		
		
		IJ.log( "KDTree Search" );
		IJ.log( "=============" );
		
		/* nearest neighbor */
		IJ.log( "Nearest neighbor ..." );
		final ImagePlusImg< UnsignedByteType, ? > img4 = factory.create( size, new UnsignedByteType() );
		t = drawNearestNeighbor(
				img4,
				new NearestNeighborSearchOnKDTree< UnsignedByteType >( kdtree ) );
		
		IJ.log( t + "ms " );
		
		try
		{
			img4.getImagePlus().show();
			IJ.log( "Done." );
		}
		catch ( final ImgLibException e )
		{
			IJ.log( "Didn't work out." );
			e.printStackTrace();
		}
		
		/* nearest neighbor through k-nearest-neighbor search */
		IJ.log( "Nearest neighbor through k-nearest-neighbor search ..."  );
		final ImagePlusImg< UnsignedByteType, ? > img5 = factory.create( size, new UnsignedByteType() );
		t = drawNearestNeighbor(
				img5,
				new KNearestNeighborSearchOnKDTree< UnsignedByteType >( kdtree, 1 ) );
		
		IJ.log( t + "ms " );
		
		try
		{
			img5.getImagePlus().show();
			IJ.log( "Done." );
		}
		catch ( final ImgLibException e )
		{
			IJ.log( "Didn't work out." );
			e.printStackTrace();
		}
		
		/* weighted by distance */
		IJ.log( "Weighted by distance ..." );
		final ImageStack distanceStack2 = new ImageStack( ( int )size[ 0 ], ( int )size[ 1 ] );
		
		np = 1;
		for ( int k = 5; k <= 30; k += 5 )
		{
			np = 0;
			t = 0;
			for ( double p = 0; p <= 2; p += 0.5, ++np )
			{
				IJ.log( "  k=" + k + " p=" + String.format( "%.2f", p ) );
				final ImagePlusImg< UnsignedByteType, ? > img3 = factory.create( size, new UnsignedByteType() );
				t += drawWeightedByDistanceInterpolator(
						img3,
						new KNearestNeighborSearchOnKDTree< UnsignedByteType >( kdtree, k ), k, p, 0, 255 );
				try
				{
					distanceStack2.addSlice( "k=" + k + " p=" + String.format( "%.2f", p ), img3.getImagePlus().getProcessor() );
				}
				catch ( final ImgLibException e )
				{
					IJ.log( "Didn't work out." );
					e.printStackTrace();
				}
			}
			
			IJ.log( t + "ms" );
		}
		
		final ImagePlus impDistance2 = new ImagePlus( "weighted by distance", distanceStack2 );
		impDistance2.setOpenAsHyperStack( true );
		impDistance2.setDimensions( 1, np, distanceStack2.getSize() / np );
		impDistance2.show();
		IJ.log( "Done." );		
	}
}
